type scriptfile;
type logfile;
type aipsdata;
type aipstable;
type aipsfile;

type aipsmetadata {
  string antennas[];
  string sources[];
}

(string antennas[]) antennas(aipsdata data) {
  aipsmetadata m;

  m = readStructured(@data);
  antennas = m.antennas;
}

(string sources[]) sources(aipsdata data) {
  aipsmetadata m;

  m = readStructured(@data);
  sources = m.sources;
}

app (aipstable table) cltable(aipsdata data) {
  touch @table;
}

app (aipstable tyver, aipstable gcver, logfile l) antab_script(scriptfile s, aipsdata indata, aipsfile calin, float offset) {
  pt @s "indata" @indata "calin" @calin "offset" offset "tyver" @tyver "gcver" @gcver stdout=@l;
}

(aipstable tyver, aipstable gcver) antab(aipsdata indata, aipsfile calin, float offset) {
  scriptfile script <"antab.py">;
  logfile log <"antab.log">;

  (tyver, gcver, log) = antab_script(script, indata=indata, calin=calin, offset=offset);
}

app (aipstable snver, logfile l) apcal_script(scriptfile s, aipsdata indata, float freqid, aipstable tyver, aipstable gcver, float prtlev) {
  pt @s "indata" @indata "freqid" freqid "tyver" @tyver "gcver" @gcver "prtlev" prtlev "snver" @snver stdout=@l;
}

(aipstable snver) apcal(aipsdata indata, float freqid, aipstable tyver, aipstable gcver, float prtlev) {
  scriptfile script <"apcal.py">;
  logfile log <"apcal.log">;

  (snver, log) = apcal_script(script, indata=indata, freqid=freqid, tyver=tyver, gcver=gcver, prtlev=prtlev);
}

app (aipstable gainuse, logfile l) clcal_script(scriptfile s, aipsdata indata, float subarray, string opcode, aipstable gainver, aipstable snver, float refant, string interpol, float doblank, float dobtween, string samptype) {
  pt @s "indata" @indata "subarray" subarray "opcode" @opcode "gainver" @gainver "snver" @snver "refant" refant "interpol" @interpol "doblank" doblank "dobtween" dobtween "samptype" @samptype "gainuse" @gainuse stdout=@l;
}

(aipstable gainuse) clcal(aipsdata indata, float subarray, string opcode, aipstable gainver, aipstable snver, float refant, string interpol, float doblank, float dobtween, string samptype) {
  scriptfile script <"clcal.py">;
  logfile log <"clcal.log">;

  (gainuse, log) = clcal_script(script, indata=indata, subarray=subarray, opcode=opcode, gainver=gainver, snver=snver, refant=refant, interpol=interpol, doblank=doblank, dobtween=dobtween, samptype=samptype);
}

app (aipsdata outdata, logfile l) fitld_script(scriptfile s, aipsfile datain, float digicor) {
  pt @s "datain" @datain "digicor" digicor "outdata" @outdata stdout=@l;
}

(aipsdata outdata) fitld(aipsfile datain, float digicor) {
  scriptfile script <"fitld.py">;
  logfile log <"fitld.log">;

  (outdata, log) = fitld_script(script, datain=datain, digicor=digicor);
}

app (aipstable snver, logfile l) fring_script(scriptfile s, aipsdata indata, float freqid, aipstable gainuse, aipstable flagver, float docalib, float subarray, float refant, float search[], float solint, string calsour[], float aparm[], float dparm[]) {
  pt @s "indata" @indata "freqid" freqid "gainuse" @gainuse "flagver" @flagver "docalib" docalib "subarray" subarray "refant" refant "search" @length(search) search "solint" solint "calsour" @length(calsour) calsour "aparm" @length(aparm) aparm "dparm" @length(dparm) dparm "snver" @snver stdout=@l;
}

(aipstable snver) fring(aipsdata indata, float freqid, aipstable gainuse, aipstable flagver, float docalib, float subarray, float refant, float search[], float solint, string calsour[], float aparm[], float dparm[]) {
  scriptfile script <"fring.py">;
  logfile log <"fring.log">;

  (snver, log) = fring_script(script, indata=indata, freqid=freqid, gainuse=gainuse, flagver=flagver, docalib=docalib, subarray=subarray, refant=refant, search=search, solint=solint, calsour=calsour, aparm=aparm, dparm=dparm);
}

app (aipsdata outdata, logfile l) imagr_script(scriptfile s, aipsdata indata, string sources[], float freqid, float subarray, float docalib, float dopol, float nchav, float chinc, string stokes, string uvwtfn, float cellsize[], float imsize[], float robust, float xtype, float ytype, float niter, float allokay, float gain, float minpatch, float imagrprm[], float dotv) {
  pt @s "indata" @indata "sources" @length(sources) sources "freqid" freqid "subarray" subarray "docalib" docalib "dopol" dopol "nchav" nchav "chinc" chinc "stokes" @stokes "uvwtfn" @uvwtfn "cellsize" @length(cellsize) cellsize "imsize" @length(imsize) imsize "robust" robust "xtype" xtype "ytype" ytype "niter" niter "allokay" allokay "gain" gain "minpatch" minpatch "imagrprm" @length(imagrprm) imagrprm "dotv" dotv "outdata" @outdata stdout=@l;
}

(aipsdata outdata) imagr(aipsdata indata, string sources[], float freqid, float subarray, float docalib, float dopol, float nchav, float chinc, string stokes, string uvwtfn, float cellsize[], float imsize[], float robust, float xtype, float ytype, float niter, float allokay, float gain, float minpatch, float imagrprm[], float dotv) {
  scriptfile script <"imagr.py">;
  logfile log <"imagr.log">;

  (outdata, log) = imagr_script(script, indata=indata, sources=sources, freqid=freqid, subarray=subarray, docalib=docalib, dopol=dopol, nchav=nchav, chinc=chinc, stokes=stokes, uvwtfn=uvwtfn, cellsize=cellsize, imsize=imsize, robust=robust, xtype=xtype, ytype=ytype, niter=niter, allokay=allokay, gain=gain, minpatch=minpatch, imagrprm=imagrprm, dotv=dotv);
}

app (aipsdata outdata, logfile l) multi_script(scriptfile s, aipsdata indata, string sources[], float aparm[]) {
  pt @s "indata" @indata "sources" @length(sources) sources "aparm" @length(aparm) aparm "outdata" @outdata stdout=@l;
}

(aipsdata outdata) multi(aipsdata indata, string sources[], float aparm[]) {
  scriptfile script <"multi.py">;
  logfile log <"multi.log">;

  (outdata, log) = multi_script(script, indata=indata, sources=sources, aparm=aparm);
}

app (aipsfile outprint, logfile l) prtab_script(scriptfile s, aipsdata indata, aipstable intable) {
  pt @s "indata" @indata "intable" @intable "outprint" @outprint stdout=@l;
}

(aipsfile outprint) prtab(aipsdata indata, aipstable intable) {
  scriptfile script <"prtab.py">;
  logfile log <"prtab.log">;

  (outprint, log) = prtab_script(script, indata=indata, intable=intable);
}

app (aipsdata outdata, logfile l) split_script(scriptfile s, aipsdata indata) {
  pt @s "indata" @indata "outdata" @outdata stdout=@l;
}

(aipsdata outdata) split(aipsdata indata) {
  scriptfile script <"split.py">;
  logfile log <"split.log">;

  (outdata, log) = split_script(script, indata=indata);
}

app (aipstable outfgver, logfile l) uvflg_script(scriptfile s, aipsdata indata, aipsfile intext) {
  pt @s "indata" @indata "intext" @intext "outfgver" @outfgver stdout=@l;
}

(aipstable outfgver) uvflg(aipsdata indata, aipsfile intext) {
  scriptfile script <"uvflg.py">;
  logfile log <"uvflg.log">;

  (outfgver, log) = uvflg_script(script, indata=indata, intext=intext);
}

app (aipsdata outdata, logfile l) uvprm_script(scriptfile s, aipsdata indata, float docalib, string stokes, float channel, float bif) {
  pt @s "indata" @indata "docalib" docalib "stokes" @stokes "channel" channel "bif" bif "outdata" @outdata stdout=@l;
}

(aipsdata outdata) uvprm(aipsdata indata, float docalib, string stokes, float channel, float bif) {
  scriptfile script <"uvprm.py">;
  logfile log <"uvprm.log">;

  (outdata, log) = uvprm_script(script, indata=indata, docalib=docalib, stokes=stokes, channel=channel, bif=bif);
}
