#! /usr/bin/env python

from AIPSTask import AIPSTask

tasks = [
    ('antab', ['tyver', 'gcver'], ['indata', 'calin', 'offset']),
    ('apcal', ['snver'],
     ['indata', 'freqid', 'tyver', 'gcver', 'prtlev']),
    ('clcal', ['gainuse'],
     ['indata', 'subarray', 'opcode', 'gainver', 'snver', 'refant',
      'interpol', 'doblank', 'dobtween', 'samptype']),
    ('fitld', ['outdata'], ['datain', 'digicor']),
    ('fring', ['snver'], ['indata', 'freqid', 'gainuse', 'flagver', 'docalib', 'subarray', 'refant', 'search', 'solint', 'calsour', 'aparm', 'dparm']),
    ('imagr', ['outdata'], ['indata', 'sources', 'freqid', 'subarray', 'docalib', 'dopol', 'nchav', 'chinc', 'stokes', 'uvwtfn', 'cellsize', 'imsize', 'robust', 'xtype', 'ytype', 'niter', 'allokay', 'gain', 'minpatch', 'imagrprm', 'dotv']),
    ('multi', ['outdata'], ['indata', 'sources', 'aparm'], 'indxr'), 
    ('prtab', ['outprint'], ['indata', 'intable']),
    ('split', ['outdata'], ['indata']),
    ('uvflg', ['outfgver'], ['indata', 'intext']),
    ('uvprm', ['outdata'], ['indata', 'docalib', 'stokes', 'channel', 'bif'])
]


header = """#! /usr/bin/env python
from AIPS import AIPS
from AIPSTask import AIPSTask
from AIPSData import AIPSUVData

import os
import sys

AIPS.userno = 1999
"""

body = """
args = sys.argv[1:]
outputs = []
tables = []
while args:
"""

trailer = """
for output in outputs:
    if output[0].exists():
        fp = open(output[1], 'w')
        idx = 0
        for antenna in output[0].antennas:
            fp.write('antennas[%d] = %s\\n' % (idx, antenna))
            idx += 1
        idx = 0
        for source in output[0].sources:
            fp.write('sources[%d] = %s\\n' % (idx, source))
            idx += 1
        fp.close()
for table in tables:
    s = table.split('_')
    if [int(s[1]), 'AIPS %s' % s[0]] in indata.tables:
        open(table, 'a')
"""

_table_adverbs = ['gainuse', 'gainver', 'gcver', 'intable', 'flagver',
                  'outfgver','snver', 'tyver']

def write_script(task, outputs, args, post=None):
    aipstask = AIPSTask(task)

    fp = open("%s.py" % task, "w")
    fp.write(header)
    fp.write("\n")
    fp.write("task = AIPSTask('%s')\n" % task)
    fp.write(body)
    for arg in outputs:
        fp.write("    if args[0] == '%s':\n" % arg)
        if arg in aipstask._data_adverbs:
            fp.write("        s = args[1].split('_')\n")
            fp.write("        data = AIPSUVData(s[0], s[1], int(s[2]), 1)\n")
            fp.write("        outputs.append((data, args[1]))\n")
            fp.write("        task.%s = data\n" % arg)
            fp.write("        if data.exists():\n")
            fp.write("            data.zap()\n")
            fp.write("            try:\n")
            fp.write("                os.remove(args[1])\n")
            fp.write("            except:\n")
            fp.write("                pass\n")
            fp.write("            pass\n")
        elif arg in _table_adverbs:
            fp.write("        s = args[1].split('_')\n")
            fp.write("        tables.append(args[1])\n")
            fp.write("        task.%s = float(s[1])\n" % arg)
        else:
            fp.write("        task.%s = args[1]\n" % arg)
            pass
        fp.write("        args = args[2:]\n")
        fp.write("        continue\n")
        continue
    for arg in args:
        fp.write("    if args[0] == '%s':\n" % arg)
        if arg in aipstask._data_adverbs:
            fp.write("        s = args[1].split('_')\n")
            fp.write("        data = AIPSUVData(s[0], s[1], int(s[2]), 1)\n")
            fp.write("        if args[0] == 'indata':\n")
            fp.write("            indata = data\n")
            fp.write("        task.%s = data\n" % arg)
        elif arg in _table_adverbs:
            fp.write("        s = args[1].split('_')\n")
            if arg == "intable":
                fp.write("        task.inext = s[0]\n")
                fp.write("        task.inver = float(s[1])\n")
            else:
                fp.write("        task.%s = float(s[1])\n" % arg)
                pass
        elif type(aipstask.__dict__[arg]) == float:
            fp.write("        task.%s = float(args[1])\n" % arg)
        elif type(aipstask.__dict__[arg]) == type(''):
            fp.write("        task.%s = args[1]\n" % arg)
        else:
            fp.write("        n = int(args[1])\n")
            fp.write("        for i in xrange(n):\n")
            if type(aipstask.__dict__[arg][1]) == float:
                fp.write("            task.%s[i + 1] = float(args[2])\n" % arg)
            else:
                fp.write("            task.%s[i + 1] = args[2]\n" % arg)
            fp.write("            args = args[1:]\n")
            fp.write("            continue\n")
            pass
        fp.write("        args = args[2:]\n")
        fp.write("        continue\n")
        continue
    fp.write("    print args\n")
    fp.write("    raise Hell\n")
    fp.write("    continue\n")
    fp.write("task.go()\n")
    if post == 'indxr':
        fp.write("""
indxr = AIPSTask('indxr')
indxr.indata = indata
indxr.cparm[1:] = [0, 22, .25]
indxr.go()
""")
    fp.write(trailer)
    fp.close()
    return

for task in tasks:
    if task[0] == 'split' or task[0] == 'imagr' or task[0] == 'uvprm':
        continue
    if len(task) > 3:
        write_script(task[0], task[1], task[2], task[3])
    else:
        write_script(task[0], task[1], task[2])
    continue

header = """type scriptfile;
type logfile;
type aipsdata;
type aipstable;
type aipsfile;

type aipsmetadata {
  string antennas[];
  string sources[];
}

(string antennas[]) antennas(aipsdata data) {
  aipsmetadata m;

  m = readStructured(@data);
  antennas = m.antennas;
}

(string sources[]) sources(aipsdata data) {
  aipsmetadata m;

  m = readStructured(@data);
  sources = m.sources;
}

app (aipstable table) cltable(aipsdata data) {
  touch @table;
}
"""

def swift_type(task, adverb):
    aipstask = AIPSTask(task)
    if adverb in aipstask._data_adverbs:
        return "aipsdata"
    if adverb in _table_adverbs:
        return "aipstable"
    if adverb in aipstask._file_adverbs:
        return "aipsfile"
    if type(aipstask.__dict__[adverb]) == float:
        return "float"
    if type(aipstask.__dict__[adverb]) == type(''):
        return "string"
    if type(aipstask.__dict__[adverb][1]) == float:
        return "floatarray"
    else:
        return "stringarray"

def write_swift(fp, task, outputs, args):
    aipstask = AIPSTask(task)
    retlist = ""
    first = True
    for output in outputs:
        if not first:
            retlist = retlist + ", "
            pass
        rettype = swift_type(task, output)
        retlist = retlist + "%s %s" % (rettype, output)
        first = False
        continue

    first = True
    arglist = ""
    for arg in args:
        if not first:
            arglist = arglist + ", "
            pass
        argtype = swift_type(task, arg)
        if argtype.endswith("array"):
            argtype = argtype.replace("array", "")
            arg = arg + "[]"
            pass
        arglist = arglist + "%s %s" % (argtype, arg)
        first = False
        continue

    fp.write("app (")
    fp.write(retlist)
    fp.write(", logfile l) %s_script(scriptfile s, " % task)
    fp.write(arglist)
    fp.write(") {\n")
    fp.write("  pt @s")
    for arg in args:
        argtype = swift_type(task, arg)
        if argtype == "float":
            fp.write(' "%s" %s' % (arg, arg))
        elif argtype.endswith("array"):
            fp.write(' "%s" @length(%s) %s' % (arg, arg, arg))
        else:
            fp.write(' "%s" @%s' % (arg, arg))
            pass
        continue
    for output in outputs:
        fp.write(' "%s" @%s' % (output, output))
    fp.write(" stdout=@l;\n")
    fp.write("}\n")

    fp.write("\n")

    fp.write("(")
    fp.write(retlist)
    fp.write(") %s(" % task)
    fp.write(arglist)
    fp.write(") {\n")
    fp.write('  scriptfile script <"%s.py">;\n' % task)
    fp.write('  logfile log <"%s.log">;\n' % task)
    fp.write("\n")
    fp.write("  (%s, log) = %s_script(script, " % (", ".join(outputs), task))
    first = True
    for arg in args:
        if not first:
            fp.write(", ")
            pass
        fp.write("%s=%s" % (arg, arg))
        first = False
        continue
    fp.write(");\n")
    fp.write("}\n")
    return

fp = open("aips.swift", "w")
fp.write(header)

for task in tasks:
    fp.write("\n")
    write_swift(fp, task[0], task[1], task[2])
    continue
