# Bojan Nikolic <b.nikolic@mrao.cam.ac.uk>
"""
Construct a DAG by doing a stubbed execution of CASA-like python code
"""
import networkx as nx

import tdesc

def uniquify(n, G):
    return "%s-%i" % (n, G.size())

def datain_f(p):
    "Data input parameters"
    return filter(lambda x : set(["in", "inout"]).intersection(set(p[x])), p)

def dataout_f(p):
    "Data input parameters"
    return filter(lambda x : set(["out", "inout"]).intersection(set(p[x])) , p)

def pars_f(p):
    "Simple (non-data) parameters"
    return filter(lambda x : not(set(["in", "inout", "out", "ms"]).intersection(set(p[x]))), p)

def buildstub(tname,
              tpars,
              G):
    """
    tpars: Task parameters
    TG   : Graph to which to add to
    """
    # below G=G because G wasn't making it into the closure
    # (late-binding of closures?)
    def fn(G=G, **kwargs):
        inl=  [(x,kwargs[x]) for x in kwargs if x in datain_f(tpars)]
        outl= [(x,kwargs[x]) for x in kwargs if x in dataout_f(tpars)]
        pars= [(x, kwargs[x])for x in kwargs if x not in ( datain_f(tpars)+dataout_f(tpars))]
        n=uniquify(tname, G)    
        G.add_node(n,
                   task=tname, 
                   pars=pars)
        for pn, pv in inl:
            if type(pv)==list:
                for xx in pv:
                    G.add_edge(xx, n, par=pn)
            else:
                G.add_edge(pv, n, par=pn)        
        # Now uniquify the outputs 
        rl={}
        for (pn, pv) in outl:
            if pv in G.nodes():
                rl[pv]=uniquify(pv, G)
        if len(rl) > 0:
            nx.relabel_nodes(G, rl, copy=False)
        # Add outputs
        for (pn, pv) in outl:
            G.add_edge(n, pv, outpar=pn)        
    return fn

def record(s, fnout):
    "Construct the DAG of  script in filename s and write to fnout"
    G=nx.DiGraph()    
    e=dict([ (x, buildstub(x, tdesc.td[x], G)) for x in tdesc.td ] + [("G", G)])
    exec open(s) in e
    nx.write_dot(G, fnout)
    

        
        
