# Bojan Nikolic <b.nikolic@mrao.cam.ac.uk>
"""Run-time engine"""

import dataset
import tempfile
import networkx as nx
import StringIO
import pickle
import os
import shutil
import subprocess

import scompile
import tdesc

casaexec="/home/bnikolic/p/casapy-42.0.28322-021-1-64b/casapy"
workdir="/home/bnikolic/temp/casaexp2/"
objstoredir="/home/bnikolic/temp/objstore"

def gtos(g):
    "Graph to string"
    o=StringIO.StringIO()
    nx.write_gexf(g, o)
    return o.getvalue()

def stog(s):
    "String to graph"
    f=tempfile.TemporaryFile()
    f.write(s)
    f.flush()
    f.seek(0)
    # Note read_gexf uses *c*ElementTree so Python StringIO does not work
    return nx.read_gexf(f)

def addInput(n, u, db):
    db['inputs'].insert(dict(name=n, url=u))

def inputnodes(g):
    return [s for s in g if  len(nx.ancestors(g,s))==0]

def outputnodes(g):
    return [s for s in g if  len(nx.descendants(g,s))==0]

def ignodes(g):
    "Return list of intermediate and output data nodes"
    return set([s for s in g if "task" not in g.node[s].keys() ]).difference(set(inputnodes(g)))

def nodedict(din):
    lin=[l.strip().split(" ") for l in  open(os.path.join(din,"datanodes.txt")).readlines()]
    return dict( [ (l[0] , l[1:]) for l in lin ])

def getNodeData(n, din, gid, tx):
    res=[]
    for nn in nodedict(din)[n]:
        fin=os.path.join(din, nn)
        if os.path.isdir(fin):
            shutil.copytree(fin, os.path.join(objstoredir, str(gid), nn ))
        else:
            shutil.copy(fin, os.path.join(objstoredir, str(gid), nn))
        res.append(tx["objs"].insert(row=dict(gid=gid,
                                              name=n,
                                              path=os.path.join(objstoredir, str(gid), nn))))
    return res

def nearestGraph(g, db):
    for ixg in db["graphs"]:
        # Using the isomorphism test here just for testing purposes
        if nx.is_isomorphic(g, stog(ixg['g'])): return   ixg['id']
    return None

def runGraph(g, db):
    """
    Add a graph to database
    """
    ngid=nearestGraph(g, db)
    if ngid:
        print "Your files are waiting for you at:"
        for d in outputnodes(g):
            print db["objs"].find_one(gid=ngid, name=d)["path"]
    else:
        with db as tx:
            if 1:
                gid=tx["graphs"].insert(row=dict(g=gtos(g)))
                s=scompile.graphBuild(g)
                casad=tempfile.mkdtemp(dir=workdir)
                shutil.copy("execenv.py", casad)
                open(os.path.join(casad, "t1.py"), "w").write(s)
                subprocess.check_call([casaexec,  "-c",  "t1.py"],
                                      cwd=casad)
        for d in ignodes(g):
            getNodeData(d, casad, gid, db)
        if 1:
            shutil.rmtree(casad)
        
        
    
    
    




