# Bojan Nikolic <b.nikolic@mrao.cam.ac.uk>
"""
Compile down to a CASA script 
"""
from collections import defaultdict
import pdb

import networkx as nx

import tdesc

def multidict(l):
    "Multiple values for same key get aggregated into a list"
    res = defaultdict(list)
    for k,v in l:
        res[k].append(v)
    return res

def delist(x):
    "Change lists of length one to just the element"
    if type(x)==list and len(x)==1:
        return x[0]
    else:
        return x

def qv(x):
    "Quote values so that they get read-back into Python in same form"
    if type(x) is list:
        return str(x)
    elif type(x) is str:
        return "'" + x + "'" 
    elif type(x) is bool:
        return str(x)
    elif type(x) is int or float:
        return str(x)
    else:
        pdb.set_trace()

def listKwArgs(d):
    return ["%s=%s" % (k,qv( delist(d[k]))) for k in d]

def printKwArgs(d):
    return ", ".join(d)

def taskCall(nn, g):
    """
    Build  call for one task 
    nn: node label
    g: graph 
    """
    res=[]
    n=g.node[nn]
    tname=n["task"]
    inp=multidict([ (g.edge[ni][no]['par'], ni)  for (ni, no) in g.in_edges(nn) ])
    outp=multidict([ (g.edge[ni][no]['outpar'], no)  for (ni, no) in g.out_edges(nn) ])
    outpu=outp.copy()
    inoutp=set(inp.keys()).intersection(outp.keys())
    pars=dict(eval(n["pars"]))
    for k in inoutp: outpu.pop(k)
    res.append( tname+"(" +printKwArgs(listKwArgs(inp)+listKwArgs(outpu) + listKwArgs(pars))+")")
    for k in inoutp:
        res.append( "shutil.copytree(%s, %s)" % (qv(inp[k][0]), qv(outp[k][0])))
    # Now output code to record info about the processing done
    res.append( "open('status.txt', 'a').write(' '.join(['%s'] + %s ) + '\\n' )" % ( nn, qv([outp[k][0] for k in outp])))
    for k in outp:
        if "prefix" in tdesc.td[tname][k]:
            res.append( "open('datanodes.txt', 'a').write(' '.join(['%s'] + glob.glob(%s+'*')) + '\\n' )" % ( outp[k][0], qv(outp[k][0])))
        else:
            res.append( "open('datanodes.txt', 'a').write(' '.join(['%s'] + [%s]) + '\\n' )" % ( outp[k][0], qv(outp[k][0])))
    return res

def graphBuild(g):
    res=["from execenv import *", "import glob"]
    def isinput(n):
        return len(nx.ancestors(g, n))==0
    for s in nx.topological_sort(g):
        n=g.node[s]
        if isinput(s):
            res.append("copyinput(%s)" % qv(s))
        elif "task" in n.keys():
            res.extend(taskCall(s, g))
    return "\n".join(res)




