Graph Representation of Data Reduction Scripts 
==============================================

It is useful to represent the data reduction scripts, and the steps
that they describe, as a graph. This allows us to apply more general
algorithms and data structures to this particular problem.

It is natural to associate the input, intermediate and output datasets
with vertices of the graph. The edges of the graph must be directed
and must at least represent the dependencies between dataset, i.e., an
edge from A to B implies that if A has changed then B must be
recomputed too. Unlike in a traditional "makefile" system, however, we
must also keep track of possibility that the operation that leads from
one dataset to another (and therefore from one vertex to another)
changes. This means that in the above example, even if A has not
changed, B needs to be recomputed if the operation which leads from A
to B has changed.

It is not entirely obvious what the natural way of representing this
operation between each vertex is. The two possibilities that come to
mind are:

1. Represent the operation as the label of the edge

2. Represent the operation to be done as a different type of vertex
   within the same graph, with edge from this vertex to the
   data-vertices that it produces. This new type of vertex could be
   called operation-vertex.
   
It could be the case that we use different representations depending
on precisely which operations on the graph we need to do. For the
purposes of dependency tracking and restarts, it seems that option 2
leads to the most straightforward implementation.

A graph constructed in this way will be a Directed Acyclic Graph
(DAG). Some radio-astronomy scripts do in fact have iteration (e.g.,
for self-calibration) which in non-functional language would be
represented with a cyclical graph. In radio-astronomy, and the
scripting language proposed, these iteration loops would effectively
be unrolled. This is not a problem since the number of iteration is
typically small and determined at compile time.

The strategy and meaning of labelling of vertices of the graph also
needs to be considered. The labelling of all root vertices (i.e.,
vertices without any edges going into them) is obviously important
since one input dataset can not, at will, be interchanged with
another. The labelling of any intermediate datasets is arbitrary in
the sense that it does not affect the result, but must be preserved
across different runs if caching is to be used. Finally, it can be
noted that using the user's variable names for intermediate vertex
labelling is useful, since this can also be used in heuristic
algorithms for graph isomorphisms. 



Notes:

1. If a script has multiple data paths to same data-vertex then this
   is likely to indicate an error since the result is non-determinate.
   We should check for this possibility to trap logic errors. 
   
