# This is the casa tutorial at:
# http://casaguides.nrao.edu/index.php?\
# title=Calibrating_a_VLA_5_GHz_continuum_survey
import os

xfl = [ "AG733_A061209.xp1", 
        "AG733_B061213.xp1"]

ampcallist = ['0137+331']
phasecallist = ['2250+143', '0119+321', '0237+288', 
                '0239-025', '0323+055', '0339-017',
                '0423-013'] 
sourcelist = ['NGC7469', 'MRK0993', 'MRK1040', 
              'NGC1056', 'NGC1068', 'NGC1194', 
              'NGC1241', 'NGC1320', 'F04385-0828',
              'NGC1667'] 
allcals = ampcallist + phasecallist
calDict = {'NGC7469':'2250+143',
           'MRK0993':'0119+321', 
           'MRK1040':'0237+288', 
           'NGC1056':'0237+288',
           'NGC1068':'0239-025',
           'NGC1194':'0323+055',
           'NGC1241':'0323+055',
           'NGC1320':'0339-017',
           'F04385-0828':'0423-013',
           'NGC1667':'0423-013'}

importvla(archivefiles=xfl,
          vis='ag733.ms')

flagcmd(vis='ag733.ms', inpmode='list', 
       inpfile=[
           "antenna='VA15' timerange='2006/12/09/04:45:10.0~04:45:20.0'",
           "antenna='VA15' timerange='2006/12/09/06:12:30.0~06:14:10.0'",
           "antenna='VA15' timerange='2006/12/13/04:32:40.0~04:35:30.0'",
           "antenna='VA15' timerange='2006/12/09/06:12:30.0~06:12:40.0'",
           "antenna='VA15' timerange='2006/12/09/06:13:00.0~06:13:40.0'",
           "antenna='VA15' timerange='2006/12/09/06:13:50.0~06:14:00.0'",
           "antenna='VA15' timerange='2006/12/09/06:14:00.0~06:14:10.0'"], 
        action='apply')

flagdata(vis='ag733.ms', 
         timerange='2006/12/09/06:24:50.0~06:25:00.0')

setjy(vis='ag733.ms', field = '0137+331', 
      modimage = '/home/bnikolic/p/VLA/CalModels/3C48_C.im')

gaincal(vis='ag733.ms',caltable='cal.G', 
        field=','.join(allcals), solint='inf',
        refant='VA05', append=False)

flagdata(vis='ag733.ms',antenna='EA26', 
         timerange='2006/12/13/0:0:0~24:0:0')
gaincal(vis='ag733.ms',caltable='cal.G', 
        field=','.join(allcals), solint='inf',
        refant='VA05', append=False)

fluxscale(vis='ag733.ms', caltable='cal.G', 
          reference=','.join(ampcallist), 
          transfer=','.join(phasecallist), 
          fluxtable='cal.Gflx', append=False)

for source, calibrator in calDict.iteritems():
    applycal(vis='ag733.ms', field=source,
             gaintable='cal.Gflx', gainfield=calibrator)

blcal(vis = 'ag733.ms', caltable = 'cal.BL', field = '0137+331', 
      solint = 'inf', gaintable = 'cal.Gflx',
      gainfield = '0137+331', interp = 'nearest', 
      timerange = '2006/12/09/0:0:0~24:0:0')

for source, calibrator in calDict.iteritems():
    applycal(vis='ag733.ms', field=source,
             gaintable=['cal.Gflx','cal.BL'], 
             gainfield=calibrator)

for source in sourcelist:
    splitfile = source + '.split.ms'
    split(vis='ag733.ms',outputvis=splitfile, 
          datacolumn='corrected', field=source)


for source in sourcelist:
    splitfile = source + '.split.ms'
    clean(vis=splitfile,
          imagename = source + '_img',
          mode      =      'mfs',
          niter               =       2000,
          gain                =        0.1,
          threshold           = '4.5e-5Jy',
          psfmode             =    'clark',  
          imagermode          =  'csclean',
          cyclefactor    =          3,
          cyclespeedup   =         -1,
          multiscale          =         [],
          interactive         =      False,
          mask                =         [],
          imsize              = [1024, 1024],
          cell                = ['0.75arcsec', '0.75arcsec'],
          stokes              =        'I',
          weighting           =  'natural',
          uvtaper             =      False,
          pbcor               =      False,
          minpb               =        0.1,
          usescratch          =      False,
          async               =      False      )

        
