module CFG (
  SNode (..)
  , getDefs
  , groupDefs
  , extractDefGroups
  , extractName
  , extractNames
  , maybeExtractName
  , iterateReaching
  , getVarUses 
  , getVars
  , getUses
  , showUses
  , showMap
  , mapOverUses
  ) where

-- their stuff:
import Control.Monad
import Data.List
import Data.Maybe
import qualified Data.Foldable as DF
import qualified Data.Map as Map
import Debug.Trace -- because I'm stuck!
-- my stuff:
import ASTPlus
import MyGraph
import FlowGraph
import DMSUtils

{- This module is poorly named; it actually calculates def-use chains,
using control flow graphs defined in the FlowGraph module.-} 

  
sameVarDef :: SNode -> SNode -> Bool
sameVarDef (Node _ (DefNode a _)) (Node _ (DefNode b _)) = a==b
sameVarDef _ _                                           = False

extractName :: SNode -> String
extractName (Node _ (DefNode a _)) = a


getVars :: SNode -> [String]
getVars n = 
  (DF.concat . (liftM flattenVarList) . extractExpression . getVal) n


isDefNode (Node _ (DefNode _ _)) = True
isDefNode _                      = False

extractNames :: [SNode] -> [String]
extractNames uses = 
  catMaybes $ map maybeExtractName uses

maybeExtractName :: SNode -> Maybe String
maybeExtractName (Node _ (DefNode a _)) = Just a
maybeExtractName _                      = Nothing


getDefs :: [SNode] -> [SNode]
getDefs nodes = filter isDefNode nodes

groupDefs :: [SNode] -> [(String, [SNode])]
groupDefs defNodes = map (\g@(x:_) -> (extractName x, g)) $ groupEquiv sameVarDef defNodes

extractDefGroups ns = (groupDefs . getDefs) ns

lookupDef :: (Eq a) => a -> [(a, [b])] -> [b]
lookupDef k l =
  case lookup k l of
    Nothing -> []
    Just m -> m

graphGenKill :: Graph StmFlowNode -> Map.Map SNode ([SNode], [SNode])
graphGenKill g =
  let
    nodes = getNodes g
    defGroups =  groupDefs $ getDefs nodes
    f n map = case n of
      Node _ (DefNode name _) -> Map.insert n ([n], lookupDef name defGroups) map
      _                       -> Map.insert n ([], []) map
  in
   foldr f Map.empty nodes

get1 n genKill =
  fromMaybe [] $ fmap fst $ Map.lookup n genKill 
    
get2 n genKill =
  fromMaybe [] $ fmap snd $ Map.lookup n genKill 

lunionAll sets = (nub . concat) sets
lunion s1 s2 = nub $ s1 ++ s2

iterateReachingAux g nodes genKill inOuts i =
  -- i is just a count of iterations; not actually needed.
  -- flag must be false to finish.
  let
    f (m, flag) n =
      let
        gen = get1 n genKill
        kill = get2 n genKill
        nin = lunionAll $ map (\p-> get2 p m) $ gpred g n :: [SNode]
        nout = lunion gen (nin \\ kill)  :: [SNode]
        nflag = (nout /= (get2 n m)) || flag :: Bool
      in
       (Map.insert n (nin, nout) m, nflag)
    (ninOuts, flag) = foldl f (inOuts, False) nodes 
  in
   case flag of
     False -> (ninOuts, flag, i)
     True -> iterateReachingAux g nodes genKill ninOuts (i+1)
   
iterateReaching :: Graph StmFlowNode -> (Map.Map SNode ([SNode], [SNode]))
iterateReaching g =
  let
    genKill = graphGenKill g
    nodes = reverse $ getNodes g
    f n map = Map.insert n ([], get1 n genKill) map
    inOuts0 = foldr f Map.empty nodes :: Map.Map SNode ([SNode], [SNode])
    (inOutMap, _, _) = iterateReachingAux g nodes genKill inOuts0 0
  in
   inOutMap

getVarUses :: Map.Map SNode ([SNode], [SNode]) -> [(String, [SNode])] -> [SNode] ->
              [(SNode, [SNode])]
getVarUses inOutMap defGroups nodes =
  let
    getExpPair :: SNode -> Maybe (SNode, Expression)
    getExpPair n@(Node _ nt) = fmap (\x ->(n, x)) $ extractExpression nt
    getUsesByName node varname =
      let
        in_ = get1 node inOutMap
        defs = lookupDef varname defGroups
      in intersect defs in_
    getUsesFromExp node exp =
      let varNames = getExpressionVars exp
      in (node, concat $ map (getUsesByName node) varNames)
    nodeExps = catMaybes $ map getExpPair nodes
  in 
   map (\(n, e) -> getUsesFromExp n e) nodeExps
            
mapOverUses :: (SNode -> b) -> [(SNode, [SNode])] -> [(b, [b])]
mapOverUses f u =
  map (\(p, cs) -> (f p, map f cs)) u

showNode (Node i _) =
  "<Node " ++ (show i) ++">"

showMap nodes m =
  let 
    f n = case Map.lookup n m of
      Nothing -> (showNode n, ([], []))
      Just (l1, l2) ->
        (showNode n, (map showNode l1, map showNode l2))
  in 
    map f nodes

getUses g =
  let
    nodes = getNodes g
    defGroups = groupDefs $ getDefs $ nodes
    reachMap = iterateReaching g 
    uses = getVarUses reachMap defGroups nodes
  in uses

showUses uses = 
  putStrLn $ unlines $ reverse $ map show uses


