module DMSUtil (
  showNewlineList
  , multiplyOut
  , accumulate
  , updatePL
  , replacePL
  , replaceIndex
  , enumerate
  , isLeft
  , almostMap
  , groupEquiv
  , pairwise
  , allDistinct
  , lookupOrDie
  , lookupAll
  , mergePLs
  ) where

import Data.List as DL
import Data.Map as DM
import Data.Foldable as DF
import Data.Maybe
import Debug.Trace -- because I'm stuck

showNewlineList us = unlines $ DL.map show us

lookupOrDie k m = fromJust $ DL.lookup k m

lookupAll k pl = DL.map snd $ DL.filter (\(k1, v1) -> k==k1) pl

multiplyOut :: [(a, [b])] -> [(a, b)]
multiplyOut l =
  DL.concat$ DL.map (\(a, bs) -> [(a, b) | b <- bs]) l

accumulate :: Ord a => [(a, b)] -> DM.Map a [b]
accumulate l =
  DL.foldr (\(k, v) m -> DM.insertWith (++) k [v] m) DM.empty l



updatePL :: (Eq k) => (v -> v) -> k -> [(k, v)] -> [(k, v)]
updatePL f k l =
  aux f k l []
  where
    aux f k0 [] r =
      let res = reverse r
      in
       res
    aux f k0 ((k, v):es) r =
      let v' = case k0 == k of
                True -> (f v)
                False -> v
      in aux f k0 es ((k,v'):r)

replacePL k v l =
  updatePL (\_->v) k l

replaceIndex :: Int -> a -> [a] -> [a]
replaceIndex i a l =
  let (before, _:after) =  splitAt i l
  in before ++ (a:after)


enumerate l = zip [1..] l

isLeft (Left _) = True
isLeft (Right _) = False


{- Sometimes you want a map, but you also want to have a boolean flag on the side.
In those cases, here you go:  -}
{- BN: This is eq to mapAccumR ? -}  
almostMap :: ((a, Bool) -> (b, Bool)) -> [a] -> Bool -> ([b], Bool)
almostMap f xs flag =
  let nf x (ys, oflag) =
        let (y, nflag) = f (x, oflag)
        in ((y:ys), nflag)
  in
   DL.foldr nf ([], flag) xs

groupAux _ grps [] = grps
groupAux f grps (x:xs) =
  let 
    (xgrp, rem) = DL.partition (\y -> f x y) xs
    group = (x:xgrp)
  in
   groupAux f (group:grps) rem

{- BN: Types please otherwise hard to read! -}
{- BN: See groupWith in GHC.Exts. If instead of (t -> t -> Bool) one
supplies Ord a => (t -> a) (e.g., a hash function), then one gets a
much more efficient algorithm and reuses a standard(ish) function
-}
groupEquiv :: (t -> t -> Bool) -> [t] -> [[t]]
groupEquiv f l =
  groupAux f [] l

pairwise l =
  zip l $ tail l

allDistinct l =
  length l == (length . nub) l

mergePLs :: (Eq a) => [(a, [b])] -> [(a, [b])] -> [(a, [b])]
mergePLs pl1 pl2 =
  let allKeys = nub $ (DL.map fst pl1) ++ (DL.map fst pl2)
  in DL.map (\k ->
              let 
                vs1 = DF.concat $ DL.lookup k pl1
                vs2 = DF.concat $ DL.lookup k pl2
              in
               (k, vs1 ++ vs2)) allKeys


{-
*DMSUtil> updatePL (\l -> (6:l)) 2 [(1, [1,2,3]), (2, [2,3,4]), (3, [])]
[(1,[1,2,3]),(2,[6,2,3,4]),(3,[])]
-}
