module Domination (
  iterateDomination
  , domTree
  , domFront
  , treeRoot
  ) where

import Data.Set as DS
import Data.Map as DM
import Data.List as DL
import Data.Maybe

import MyGraph -- for Node
import SNode
import TupleGraph
import Debug.Trace -- because I'm stuck

maybeToSet s =
  case s of
   Nothing -> DS.empty
   Just s -> s

lookupSet k map = maybeToSet $ DM.lookup k map

-- treeRoot :: Ord a => [(a, DS.Set a)] -> a
treeRoot t =
  let
    l = DM.toList t
    lhss = DS.fromList $ DL.map fst l
    rhss = DL.foldl1 DS.union $ DL.map snd l
    [r] = DS.toList (lhss DS.\\ rhss)
  in r

iterateDomination g =
  let 
    nodes = DL.map fst $ getNodes g
    doms0 = DM.fromList [(n, DS.fromList nodes) | n <- nodes]
  in
   iterateDominationAux g nodes doms0
  where
    iterateDominationAux g nodes doms =
      let 
        getDoms e m = lookupSet e doms
        f (flag, m) n =
          let oldS =  getDoms n m
              -- gpred => [Integer]
              preds = gpred g n
              newS = case Prelude.null preds of
                True  -> DS.singleton n
                False ->
                  (DS.singleton n) `DS.union` foldl1 DS.intersection [getDoms p m | p <- preds]
              m' = DM.insert n newS m
          in (flag || (newS /= oldS), m')
        (changed, m') = Prelude.foldl f (False, doms) nodes
      in
       case changed of
        True -> iterateDominationAux g nodes m'
        False -> m'

domTree g = 
  -- given the complete map of dominators of each node, we want to
  -- extract only the *immediate* dominator, so that we can form the
  -- dominator tree.  The immediate dominator, i, is the dominator
  -- such that i dominates the node, n, but i does not dominate any of
  -- n's dominators except i and n.  In practice that means that the
  -- immediate dominator is the node (if any) that dominates one less
  -- node that the node n. (I think.)
  let m = iterateDomination g
      idomMaybes = 
        DM.mapWithKey (\k v ->
                        let n = DS.size v
                            idoml = DS.filter (\d -> let ds = DS.size (lookupSet d m)
                                                     in ds == n-1) v
                            idom = case DS.toList idoml of
                              [e] -> Just e
                              _   -> Nothing
                        in idom) m
        in
   DM.toList $ DM.map fromJust $ DM.filter isJust idomMaybes 

domFront idom cfge doms =
  case roots idom of
    [] -> DM.empty
    [r] ->
      trace ("domFront root" ++ show r) $ computeDF DM.empty r
  where
    computeDF dfMap n =
      let
        successors =  successor n
        childrens = children n
        dfLocal = DS.fromList $ DL.filter (\y -> not $ isIdom y n) $ successors
        mChildren = DL.foldl computeDF dfMap $ childrens
        ndoms = (fromJust $ DM.lookup n doms)
        dfUp = DS.fromList [w | c <- children n,
                            w <- DS.toList $ fromJust $ DM.lookup c mChildren,
                            not $ DS.member n ndoms]
        dfMap' = DM.insert n (dfLocal `DS.union` dfUp) mChildren
      in
       -- trace (unlines $ ["[domFront] node: "++show n 
       --                  , "childrens: "++ show childrens
       --                  , "successors " ++ show successors
       --                  , "dfLocal: " ++ show dfLocal
       --                  , "mChildren: " ++ show mChildren
       --                  , "dfMap': "++ show dfMap'
       --                  , "dfUp: "  ++ show dfUp
       --                  , "ndoms: " ++ show ndoms
       --                  ])
       dfMap'
    successor n = succs cfge n
    parents n = preds idom n 
    children n = succs idom n
    isIdom n w = ([w] == parents n)
