module EditList (
  EditOp (RenameNode, InsertNode, DeleteNode, InsertEdge, DeleteEdge)
  , applyEdit
  , applyEditList
  ) where

import Data.List

-- We would like to also have "AlterNode", but that is going to be
-- hard: for different types, it may involve different operations.
data EditOp a = RenameNode a a
              | InsertNode a [a]
              | DeleteNode a
              | InsertEdge a a
              | DeleteEdge a a deriving (Show)
                
applyRename oldName newName g =
  map (\(parent, child) -> (rename(parent), rename(child))) g
  where rename n 
          | n==oldName = newName
          | otherwise  = n

applyInsertNode node parents g =
  [(p, node) | p<-parents] ++ g

applyDeleteNode node g =
  filter (\(p,c) -> p/=node && c/=node) g

applyInsertEdge a b g =
  (a, b) : g

applyDeleteEdge a b g = 
  delete (a, b) g

applyEdit edit g =
   case edit of
     RenameNode a b ->
       applyRename a b g
     InsertNode a parents ->
       applyInsertNode a parents g
     DeleteNode a ->
       applyDeleteNode a g
     InsertEdge a b ->
       applyInsertEdge a b g
     DeleteEdge a b ->
       applyDeleteEdge a b g
       
applyEditList edits g =
  foldr applyEdit g edits
