module FlowGraph (
  SNode
  , StmFlowNode (..)
  , StmFlowNode0 (..)
  , BasicItem (..)
  , FlowNodePair
  , extractItemExpression
  , extractItemDefVar
  , extractExpression
  , extractItems
  , replaceItems
  , extractDefVar
  , addTupleEdge
  , addTupleEdges
  , fromTreeGraph
  , mergeEdges
  ) where

import DMSUtil
-- import Control.Exception -- assert
import Data.List -- nub
-- import Data.Fix
import Data.Maybe
import Expression
import ASTPlus
import MyGraph
import TreeGraph
import Tree (StmNode (..)) -- just for StmNode
import Debug.Trace -- because I'm stuck!



type SNode = Node StmFlowNode 

{- Code for the control flow graph (CFG) for a syntax tree. Again, tightly coupled to the syntax.
-}

data BasicItem a =  ExpNode a
                 | DefNode String a
                 | PhiNode String a
                 deriving (Eq, Ord)
                                             

extractItemDefVar (DefNode n _) = Just n
extractItemDefVar (PhiNode n _) = Just n
extractItemDefVar _ = Nothing

extractDefVar b = catMaybes $ map extractItemDefVar (extractItems b)

extractItemExpression s =
  case s of
   (ExpNode e) -> e
   (DefNode _ e) -> e
   (PhiNode _ e) -> e

instance Show a => Show (BasicItem a) where
  show (ExpNode e) = "ExpNode "++(show e)
  show (DefNode n v) = "DefNode "++n++" := "++(show v)
  show (PhiNode n v) = "PhiNode "++n++" := "++(show v)

data StmFlowNode0 a =
  BlockNode [BasicItem a]
  | IfTest a
  | IfEnd [BasicItem a]
  | DoStart
  | DoEnd a deriving (Eq, Ord)

type StmFlowNode = StmFlowNode0 Expression 

instance Show a => Show (StmFlowNode0 a) where
  show (BlockNode l) = "BlockNode " ++ (show l) 
  show (IfTest e) = "IfTest "++(show e)
  show (IfEnd l) = "IfEnd " ++ (show l)
  show DoStart = "DoStart"
  show (DoEnd e) = "DoEnd while "++show e

type FlowNodePair = (Node StmFlowNode, Node StmFlowNode)

extractExpression :: StmFlowNode -> [Expression]
extractExpression s =
  case s of 
    BlockNode l -> map extractItemExpression l
    IfEnd l -> map extractItemExpression l
    IfTest e -> [e]
    DoEnd e  ->  [e]
    _  -> []

extractItems s = 
  case s of 
    BlockNode l -> l
    IfEnd l -> l
    _  -> []

replaceItems s [] =  s
replaceItems s l@(_:_) =
  case s of
   BlockNode _ -> BlockNode l
   IfEnd _     -> IfEnd l


addTupleEdge (n1, n2) g = addEdge g n1 n2
addTupleEdges edgs g = foldr addTupleEdge g edgs


-- result is CFG + inOutMap + gSynMap.  In merging basic blocks, the
-- first one will be reduced to basic blocks; the second needs to be.
-- The inOutMap is what Harrold et al call a Control Dependency Graph,
-- as distinct from a Control Flow Graph.  I'm not sure where I got
-- the algorithm; I'm also not sure if I ever used it.

fromTreeGraph :: [(Integer, Integer)] -> [(Integer, StmNode)] ->
                 (Graph StmFlowNode, [(Integer, FlowNodePair)], [(Integer, [Integer])])
fromTreeGraph g m =
  foldl foldFun (makeGraph, [], []) $ postOrderG g
  where
    foldFun :: (Graph StmFlowNode, [(Integer, FlowNodePair)], [(Integer, [Integer])]) -> Integer ->
               (Graph StmFlowNode, [(Integer, FlowNodePair)], [(Integer, [Integer])])
    foldFun (cfg, inOutMap, graphSynMap) i =
      let
        children = findChildren i g
        Just sn = lookup i m
      in
      case sn of
        ExpStNode e ->
          let (cfg1, n) = addNode cfg $ BlockNode [ExpNode e]
          in (cfg1, (i, (n, n)):inOutMap, (toInteger (getInt n),[i]):graphSynMap)
        AssignNode name v ->
          let (cfg1, n) = addNode cfg $ BlockNode [DefNode name v]
          in (cfg1, (i, (n, n)):inOutMap, (toInteger (getInt n),[i]):graphSynMap)
        CompoundNode ->
          let -- This very clumsy even if it is right
            c1 = head children
            cN = last children
            (cfg'', inOutMap'') =
              foldr (\(l, r) (cfg', inOutMap') ->
                      let
                        Just (_, out1)  = lookup l inOutMap'
                        Just (in2, _ ) = lookup r inOutMap'
                      in
                       (addEdge cfg' out1 in2, inOutMap')) (cfg, inOutMap) (pairwise children)
            Just (cin, _) = lookup c1 inOutMap''
            Just (_, cout) = lookup cN inOutMap''
          in
           (cfg'', (i, (cin, cout)):inOutMap, graphSynMap)
        IfNode te ->
          let
            [th, el] = children
            Just (inthen, outthen) = lookup th inOutMap
            Just (inelse, outelse) = lookup el inOutMap
            (cfg1, inif) = addNode cfg $ IfTest te
            (cfg2, outif) = addNode cfg1 (IfEnd [])
            cfgOut = foldr addTupleEdge cfg2 [(inif, inthen), (inif, inelse)
                                             , (outthen, outif), (outelse, outif)]
          in (cfgOut, (i, (inif, outif)):inOutMap, ((toInteger $ getInt inif),[i]):graphSynMap)
        DoNode e ->
          let
            [b] = children
            Just (inbody, outbody) = lookup b inOutMap
            (cfg1, indo) = addNode cfg DoStart
            (cfg2, outdo) = addNode cfg1 $ DoEnd e
            cfgOut = foldr addTupleEdge cfg2 [(indo, inbody), (outbody, outdo) , (outdo, indo)]
          in (cfgOut, (i, (indo, outdo)):inOutMap, ((toInteger $ getInt outdo),[i]):graphSynMap)


-- the Dominance Frontier algorithm we're using apparently doesn't
-- work unless you really do group statements into "basic blocks".
-- Ideally I'd find out why and make it work anyway, but in practice I
-- am doing the grouping into basic blocks.
mergeEdges cfg =
  let edges0 = edges cfg
      nodes0 = nodes cfg
      (bNodes, other) = partition joinsBlocks edges0
      blockHeads = heads bNodes
      chains = map (\h -> makeChain h [] bNodes) blockHeads 
      merged = map (\c -> mergeChain c) chains :: [((Integer, StmFlowNode), [Integer])]
      edges' = foldl (\o m -> replaceMergedAwayEdges o m) other merged 
      nodeNumbers' = nub $ (map fst edges') ++ (map snd edges')
      mergedNodeNumbers = map fst (map fst merged)
      unmergedNodeNumbers = [i | i <- nodeNumbers', not (elem i mergedNodeNumbers)]
      nodes' = (map fst merged) ++ (map (\i -> (i, fromJust $ lookup i nodes0)) unmergedNodeNumbers)
      maxInt = foldl max 0 (map getInt nodes')
      cfg' = Graph{nodeCount=maxInt, nodes=nodes', edges=edges'}
  in trace ("Chains " ++ show chains) $
     trace ("Merged " ++ show merged) $
     trace ("mergedNodeNumbers " ++ show mergedNodeNumbers) $
     trace ("nodes' " ++ show nodes')
     trace ("cfg' " ++ show cfg')
     cfg'
  where joinsBlocks (i1, i2) =
          let ns = nodes cfg
              n1 = fromJust $ lookup i1 ns
              n2 = fromJust $ lookup i2 ns
          in
          case (n1, n2) of
           (BlockNode _, BlockNode _) -> True
           _                          -> False
        heads :: Eq a => [(a, a)] -> [a]
        heads l = (map fst l) \\ (map snd l)
        makeChain h res bNodes =
          case lookup h bNodes of
           Just next -> makeChain next (h:res) bNodes
           Nothing -> (h:res)
        -- mergeChain :: [(Integer, SNode)] -> [Integer] -> ([(Integer, SNode)], [Integer])
        mergeChain inds =
          let ns = nodes cfg
              bs = concat $ map (\i ->
                                  let BlockNode b = fromJust $ lookup i ns
                                  in b) inds
          in ((head inds, BlockNode $ reverse bs), tail inds)
        replaceMergedAwayEdges others ((i, _), js) =
          -- replace all instances of any of the js with Node i
          map (\(j1, j2) ->
                let j1' = if j1 `elem` js then i else j1
                    j2' = if j2 `elem` js then i else j2
                in (j1', j2')) others

