module FromCache (
  matchFiles
  , makeSubstitutions
  , substituteVars
  ) where

import Data.List as DL
import ASTPlus (VarName)
import Expression
import MyGraph
import FlowGraph
import DMSUtil
import Debug.Trace -- because I'm stuck

matchFiles fnNodeRanges varDupes =
  let
    -- fill out the full set of indices mapping into their files.
    indexMap = [(j, fn) | (fn, (j0, j1)) <- fnNodeRanges,
                j <- [j0..(j1-1)]]
  in
   DL.map (\l -> DL.map (\n@(i, Just varname, _) ->
                          let fn = lookupOrDie i indexMap
                          in (fn, varname)) l) varDupes


{-

We are going to replace variables in fn with definitions from any other
file. In our practice, the priviledged file will be the last one.

-}
makeSubstitutions :: [[(FilePath, VarName)]] -> FilePath -> [(VarName, (FilePath, VarName))]
makeSubstitutions fileMapList fn =
  concat $ map handleGroup fileMapList
  where
    handleGroup :: [(FilePath, VarName)] -> [(VarName, (FilePath, VarName))]
    handleGroup fileVarList =
      let (sources, dests) = partition ((==fn) . fst) fileVarList
          -- it is imaginable that we have a catalogue to look these up in,
          -- but for at least now we assume the first one is as good as any.
          prefSource = head sources
      in
       trace ("Source/dests " ++ show sources ++ ", " ++ show dests) $
       [(d, prefSource) | (_, d) <- dests]


substituteVars cfg subs =
  let ns = nodes cfg
      ns2 = map subNode ns
  in
   cfg{nodes=ns2}
  where
    subNode (i, n) =
      let items = extractItems n
          items' = map subItem items
          n' = replaceItems n items'
      in (i, n')
    subItem item@(DefNode v e) =
      let item' = case lookup v subs of
            Just (file, varname) ->
              DefNode v $ ProcCallExp "var_from_cache" [KeyArg "var" $ StringExp varname,
                                                        KeyArg "file" $ StringExp file]
            Nothing -> item
      in trace ("looking up " ++ v ++ " in " ++ show subs) $
         trace ("Item " ++ show item ++ " -> " ++ show item') $
         item' 
    subVar item = item
          
