module MyGraph (
  Node (..)
  , getInt
  , getVal
  , Graph (..)
  , makeGraph
  , addNode
  , lookupNode
  , addEdge
  , getNodes
  , gpred
  , gsucc
  , removeNode
  , removeNodes
  , replaceNode
  ) where

{- adapting design from Appel's compiler book. -}
import TupleGraph
import Data.Maybe

type Node a = (Integer, a) -- deriving (Eq, Ord, Show) 
data Graph a = Graph{nodeCount::Integer, nodes::[Node a], edges::[(Integer, Integer)]}

instance Show a => Show (Graph a) where
  show (Graph i nodes edges) =
    let
      showNode ((i, _)) = "<Node " ++ (show i) ++ ">"
      showEdge (i1, i2) = (show i1) ++ " -> " ++ (show i2)
    in
     unlines $ ["Nodes = "] ++ (reverse $ map show nodes) ++ ["Edges = "] ++ (map showEdge edges)
    
getInt :: Node a -> Integer
getInt ((i, _)) = toInteger i

getVal :: Node a -> a
getVal ((_, v)) = v

makeGraph = Graph 0 [] []

addNode (Graph i nodes edges) val =
  let node = (i, val)
  in (Graph (i+1) (node:nodes) edges, node)

lookupNode (Graph _ nodes _) i = fromJust $ lookup i nodes

replaceNode (Graph m nodes edges) (i, n) =
  let nodes' = filter (\(j, _) -> j /= i) nodes
      nodes'' = (i, n):nodes'
  in (Graph m nodes'' edges)

removeNode n (Graph i nodes edges) =
  let
    epred (n, _) (n1, n2) =
      (n /= n1) && (n /= n2)
    nodes' = filter (\n' -> n /= n') nodes
    edges' = filter (epred n) edges
  in Graph i nodes' edges'

removeNodes g ns =
  foldr removeNode g ns

getNodes :: Graph a -> [Node a]
getNodes (Graph _ nodes _) = nodes

-- by convention, edges are *directed*
addEdge (Graph i nodes edges) (i1, _) (i2, _) =
  let edge = (i1, i2)
  in Graph i nodes (edge:edges)

gpred (Graph _ _ edges) node = 
  preds edges node

gsucc (Graph _ _ edges) node =
  succs edges node
