module SSA (
  insertPhis
  , insertPhiNodes
  , renameVariables
  ) where

import Data.List as DL
import Data.Maybe
import Data.Map as DM
import qualified Data.Set as DS
import Expression
import ASTPlus
import MyGraph
import TreeGraph
import TupleGraph
import FlowGraph
import Debug.Trace -- because I'm stuck
import DMSUtil




insertPhiNodes v l cfg =
  -- trace ("insertPhiNodes " ++ v ++ " " ++ (show l)) $
  DL.foldl (\cfg' e -> insertPhiNode v e cfg') cfg l

insertPhiNode v i cfg =
  trace ("insertPhiNode " ++ v ++ " " ++ (show $ length $ gpred cfg i)) $
  let
    n = lookupNode cfg i
    l = extractItems n
    nargs = length $ gpred cfg i
    args = replicate nargs (PosArg (VarExp v))
    item = PhiNode v (ProcCallExp ("phi_" ++ show i) args)
    n' = replaceItems n (item:l)
    cfg' = (replaceNode cfg (i, n'))
  in trace ("insertPhiNode "++ v ++ " " ++ (show nargs)) $
     trace (show cfg') $
     cfg'
     
-- Deprecated. We now allow statements in EndIf nodes, and probably we will soon allow them in EndDo as well
movePhisToBlockNodes :: [(String, DS.Set Integer)] -> Graph StmFlowNode -> [(String, [Integer])]
movePhisToBlockNodes phimap cfg =
  DL.map movePhisToBlockNodesV phimap
  where
    movePhisToBlockNodesV :: (String, DS.Set Integer) -> (String, [Integer])
    movePhisToBlockNodesV (v, phis) =
      let newNodes = DL.map moveToBN $ DS.toList phis
      in (v, newNodes)
    moveToBN n =
      case lookupNode cfg n of
       BlockNode _ -> n
       _ -> let [n'] = gsucc cfg n
            in moveToBN n'

-- modeled on Cytron et al's Figure  11 (p.20 or p.470 depending on counting)

insertPhis :: [String] -> [(String, [Integer])] -> Map Integer (DS.Set Integer)
              -> [(String, (DS.Set Integer))]
insertPhis vs varMap domF =
  DL.map insertPhisSingle vs
  where
    insertPhisSingle v =
      trace ("Doing variable " ++ (show v)) $ 
      let w = lookupOrDie v varMap
      in insertPhisSingleAux v w DS.empty
    -- insertPhisSingleAux :: String -> [SNode] -> DS.Set SNode -> (String, DS.Set SNode)
    insertPhisSingleAux v []  already =
      trace ("End of insertPhisSingleAux (v, already)" ++ (show (v, already))) $ 
      (v, already)
    insertPhisSingleAux v (x:ws) already = 
      let
        (ws', already') = innerLoop x ws already
      in insertPhisSingleAux v ws' already'
    innerLoop x w0 already0 =
      let doms = (case DM.lookup x domF of
                   Nothing -> []
                   Just t -> DS.toList t)
      in
       trace ("Top of inner loop, (x, w0, already0) -> " ++ (show (x, w0, already0))) $
       trace ("Doms ->" ++ (show doms) ++ ", " ++ " already0 " ++ (show already0)) $
      DL.foldl (\(ws, already) y  ->
                 trace ("innerloop1 (ws, already): " ++ show (ws, already)) $ 
                 trace ("innerloop1 y: " ++ show y) $ 
                 case (y `DS.member` already) of
                  True -> (ws, already)
                  False ->
                    let already' = y `DS.insert` already
                        ws' = ws ++ [y]
                    in trace ("innerloop2 (ws', already'): " ++ show (ws', already')) $
                       (ws', already')) (w0, already0) doms

{-
Recall from ASTPlus that:

data Expression = ProcCallExp String [Arg]
                | VarExp String
                | FloatExp Double
                | IntExp Integer
                | UnaExp UnaOp Expression
                | BinExp BinOp Expression Expression
                | StringExp String
                deriving (Eq, Ord)

data Arg = PosArg Expression
          | KeyArg String Expression
          deriving (Eq, Ord)
-}


renameVarsInArg (PosArg e) stacks = PosArg (renameVarsInExp e stacks)
renameVarsInArg (KeyArg s e) stacks = KeyArg s (renameVarsInExp e stacks)

-- this should be a fixed point type and an easy definition. i am
-- stupid stressed and tired, so it isn't.
renameVarsInExp (ProcCallExp s args) stacks =
  ProcCallExp s (DL.map (\a-> renameVarsInArg a stacks) args)
renameVarsInExp (VarExp v) stacks =
  let (i:_) = lookupOrDie v stacks
  in
   trace ("renameVarsInExp: Looking up " ++ v ++ " in " ++ (show stacks)) $
   (VarExp (v++"_"++(show i)))
renameVarsInExp e@(FloatExp _) stacks = e
renameVarsInExp e@(IntExp _) stacks = e
renameVarsInExp e@(StringExp _) stacks = e
renameVarsInExp (UnaExp op e) stacks = UnaExp op (renameVarsInExp e stacks)
renameVarsInExp (BinExp op e1 e2) stacks =
  BinExp op (renameVarsInExp e1 stacks) (renameVarsInExp e2 stacks)

bumpDefs (stacks, counts) items =
  let 
    ((stacks', counts'), items') = DL.foldl aux ((stacks, counts), []) items
  in
   trace ("bumpDefs " ++ show items) $
   trace ("bumpDefs " ++ show items') $
   trace ("bumpDefs counts' " ++ show counts') $
   trace ("bumpDefs stacks' " ++ show stacks') $
   ((stacks', counts'), reverse items')
  where
    aux ((stacks, counts), items) item =
      let item' = replaceNonPhiUses item stacks
      in
       trace ("aux " ++ show items) $
       case item' of
         (DefNode v e) ->
           let
             (v', (stacks', counts')) = makeNewV v (stacks, counts)
           in ((stacks', counts'), (DefNode v' e):items)
         (PhiNode v e) -> 
           let (v', (stacks', counts')) = makeNewV v (stacks, counts)
           in ((stacks', counts'), (PhiNode v' e):items)
         _ ->
           ((stacks, counts), item':items)
    makeNewV v (stacks, counts) = 
      let counts' = updatePL (+1) v counts 
          i = lookupOrDie v counts'
          stacks' = updatePL (i:) v stacks
          v' = v ++ "_" ++ (show i)
      in (v', (stacks', counts'))


replaceDummyPhis :: (BasicItem Expression) -> Integer -> [(String, [Integer])]
                    -> (BasicItem Expression)
replaceDummyPhis (PhiNode vlhs (ProcCallExp phi args)) j stacks =
  -- we have defined phi nodes as expressions, where they are function
  -- calls to a function phi with only positional arguments of all the
  -- variables.
  let
    (PosArg (VarExp v)) = args !! (fromInteger j) -- very assumptious, but this had all better be true!
    (i:_) = lookupOrDie v stacks
    v' = v++"_"++(show i)
    args' = replaceIndex (fromInteger j) (PosArg (VarExp v')) args
  in
   trace ("replaceDummyPhis: v: " ++  v ++ " v': " ++ v')
   trace (" i: " ++ show i ++ " j: " ++ show j ++ " args: " ++ show args) $
   trace ("stacks: " ++ show stacks) $
   (PhiNode vlhs (ProcCallExp phi args'))
replaceDummyPhis n _ _ = n


popVars vars stack0 =
  let stack' = DL.foldl (\stack v -> updatePL pop v stack) stack0 vars
  in stack'
  where pop (h:t) = t


replaceNonPhiUses (ExpNode e) stacks = ExpNode (renameVarsInExp e stacks)
replaceNonPhiUses (DefNode n e) stacks = DefNode n (renameVarsInExp e stacks)
replaceNonPhiUses n@(PhiNode _ _) _ = n


-- Algorithm 19.7 in Big Tiger book p.437
renameVariables vs cfg domTree =
  let
    -- FIXME: fails if domTree is empty. 
    nodes = getNodes cfg -- nodes of type (Integer, a) where a is SNone (=StmFlowNode0 Expression)
    root = case findRoots domTree of
      [r] -> r
      [] -> fst $ head nodes
    indices = DL.map getInt nodes
    stacks0 = [(v, [0]) | v <- vs]
    counts0 = [(v, 0) | v <- vs]
    es = edges cfg
    nC = nodeCount cfg
    -- fixme: this is not supposed to be a fold! it handles its own recursions!
    ((stacks', counts'), nodes') = rename ((stacks0, counts0), nodes) root
  in
   (Graph nC nodes' es)
  where
    rename ((stacks0, counts0), nodes0) nodeIndex =
      let
        n0 = lookupOrDie nodeIndex nodes0
        vars = extractDefVar n0
        items0 = extractItems n0
        ((stacks2, counts2), items2) = bumpDefs (stacks0, counts0) items0
        nodes2 = replacePL nodeIndex (replaceItems n0 items2) nodes0
        successors = (gsucc cfg nodeIndex)
        nodes3 = DL.foldl (\nodes y -> doPhis (nodes, y) nodeIndex stacks2)
                 nodes2 (enumerate successors)
        children = succs domTree nodeIndex
        ((stacks4, counts4), nodes4) = DL.foldl rename ((stacks2, counts2), nodes3) children
        stacks5 = popVars vars stacks4
      in
       trace ("items0 "++ show items0) $
       trace ("domTree " ++ show domTree) $
       trace ("rename " ++ show nodeIndex) $
       -- trace ("items1 "++ show items1) $
       trace ("items2 "++ show items2) $
       trace ("\nrenameVariables: " ++ "Node " ++ show nodeIndex) $ 
       trace ("nodes0 " ++ show nodes0) $
       trace ("nodes2 " ++ show nodes2) $ 
       trace ("nodes4 " ++ show nodes4) $ 
       ((stacks5, counts4), nodes4)
    doPhis (nodes, (i, succIndex)) n stacks =
      let
        ynode = lookupOrDie succIndex nodes
        es = edges cfg
        j = toInteger $ fromJust $ elemIndex n (gpred cfg succIndex)
        items = extractItems ynode 
        items' = trace ("doPhis: " ++ show n ++ " " ++ show (gpred cfg succIndex)) $
                 trace (" -> " ++ (show j) ++ " " ++ (show ynode)++"\n") $
                 DL.map (\item -> replaceDummyPhis item j stacks) items
        sn' = replaceItems ynode items'
        nodes' = replacePL succIndex sn' nodes
      in
       nodes'


