module StmNode (
  StmNode (..)
  , Label (..)
  , getLabel
  , showValue
  , extractNodeMap
  )
where

import Data.Fix
import Expression
import ASTPlus
  
-- The point of StmNode is that unlike the AST node type it just
-- contains the expression itself, with no reference to its children.
data StmNode = CompoundNode
             | ExpStNode Expression
             | AssignNode String Expression
             | IfNode Expression
             | DoNode Expression
             deriving (Eq, Ord, Show)

-- We need to be able to compare expressions by type, so we need to be able to extract the node type.
data Label = CompoundLabel
           | ExpStLabel
           | AssignLabel
           | IfLabel
           | DoLabel
           deriving Eq

getLabel :: StmNode -> Label
getLabel s =
  case s of
    CompoundNode     -> CompoundLabel
    ; ExpStNode _    -> ExpStLabel
    ; AssignNode _ _ -> AssignLabel
    ; IfNode _       -> IfLabel
    ; DoNode _       -> DoLabel

showValue sn =
  case sn of
    ExpStNode e         -> Prelude.show e
    ; AssignNode name e -> name ++ ":=" ++ Prelude.show e
    -- It turns out we need also "values" (i.e., strings) for inner nodes
    ; CompoundNode      -> "compoundNode"
    ; IfNode e          -> "if (" ++ (Prelude.show e) ++ ")"
    ; DoNode e          -> "do (" ++ (Prelude.show e) ++ ")"

-- extractNodeMap turns an augmented tree into a series of individual
-- atomic nodes indexed by the augmented component, which is always
-- going to be an integer in practice.
extractNodeMap :: StatementPlus a -> [(a, StmNode)]
extractNodeMap t =
  case t of
    Fix (ExpStPlus i e) -> [(i, ExpStNode e)]
    Fix (CompoundPlus i tl tr) ->
      [(i, CompoundNode)] ++ (extractNodeMap tl) ++ (extractNodeMap tr)
    Fix (AssignPlus i name e) ->
      [(i, AssignNode name e)]
    Fix (IfPlus i e tl tr) ->
      [(i, IfNode e)] ++ (extractNodeMap tl) ++ (extractNodeMap tr)
    Fix (DoPlus i e t)     ->
      [(i, DoNode e)] ++ (extractNodeMap t)
