module TopSort (
  getTopSortedNodesAsList
  , getTopSortedNodesAsEither
  , topSortFromRoots
  , gFromList
  , dfs) where

import Data.Char
import qualified Data.Map as Map
import Data.List (nub, sort)

-- BN: This is already implemented in FGL as topsort
-- topsort ( mkGraph (zip [1..5] "abcde") [(1, 2, ()), (1,3, ()), (2,4,()), (3,4,()) ] :: Gr Char ())

{-

We define our graph as a Map from Nodes (strings in the first
instance) to predecessor counts and lists of successors.

I'm borrowing heavily from _The Awk Programming Language_, by Aho,
Weinberg and Kernighan (pp.172-174) for rudiments of topological
sorting.

-}

getTopSortedNodesAsEither root g = 
  dfs root $ gFromList g

-- getTopSortedNodesAsList is VERY BAD in that it silently turns
-- errors (that your graph is cyclical) into empty lists.
-- VERY VERY BAD.
getTopSortedNodesAsList :: Ord a => a -> [(a, a)] -> [a]
getTopSortedNodesAsList root g =
  eitherToList $ dfs root $ gFromList g
  where 
    eitherToList (Left  _) = []
    eitherToList (Right l) = l

topSortFromRoots :: Ord a => [a] -> [(a, a)] -> [a]
topSortFromRoots roots g =
  let
    -- the idea is simply to list the topsorts of different roots so
    -- that dependencies come *first*. Then we can concatenate and
    -- just select the first time a dependency occurs with nub.  The
    -- whole thing's a hack - we should really fix dfs itself, which
    -- wouldn't even be that hard.
    s = map (\r -> reverse $ getTopSortedNodesAsList r g) roots
  in
   (reverse . nub . concat) s
  
gFromList l =
  foldl (\m (p, s) -> Map.alter (update s) p m) Map.empty l
  where update s Nothing       = Just (1, [s])
        update s (Just (c, l)) = Just (c+1, s : l)

data Visited = NotVisited | InProgress | Finished deriving (Enum, Show)

dfs node graph =
  let children n =
        sort $ snd $ Map.findWithDefault (0,[]) n graph
      dfs_full n (Right (visitedMap, result)) =
        let
          handleChild acc@(Right (vm, r)) suc =
            case (Map.findWithDefault NotVisited suc vm) of
              NotVisited -> dfs_full suc acc
              Finished   -> acc
              InProgress -> Left ("Cycle", n, suc)
          handleChild acc _ = acc
          acc0 = (Right (Map.insert n InProgress visitedMap, result))
          newAcc = foldl handleChild acc0 (children n)
        in
         case newAcc of
           (Right (vm', r')) -> Right (Map.insert n Finished vm', n:r')
           _                 -> newAcc
      dfs_full n acc = acc
  in
   case dfs_full node (Right (Map.empty, [])) of
     Right (_, path)   -> Right path
     Left error        -> Left error
     
