module ToyParser (
  parseString
  , parseFile
  ) where

import Text.ParserCombinators.Parsec
import qualified Text.ParserCombinators.Parsec.Token as PT
import Debug.Trace -- because I'm stuck!

{-
Real World Haskell became annoying before it became useful, so I'm going to try plagiarising 
<http://www.haskell.org/haskellwiki/Parsing_expressions_and_statements> instead.

That only handles boolean values; I don't want those, but I do want integers.
-}


import Control.Applicative((<*))
-- try can be Text.ParserCombinators.Parsec.try or Text.Parsec.try;
-- we use the former.
import Text.Parsec hiding (try)
import Text.Parsec.String
import Text.Parsec.Expr
import Text.Parsec.Token
import Text.Parsec.Language
import Data.Fix
import Expression
import ASTPlus 

-- we have to translate compound statements from sensible lists into our misguided tree form;
-- in later processing they will be translated back to lists.
compoundStms [s] = s
compoundStms [s1, s2] =
  Fix $ CompoundStm s1 s2
compoundStms (s:ss) =
  Fix $ CompoundStm s (compoundStms ss)

def = emptyDef{commentStart = "#"
              , commentEnd = ""
              , identStart = letter
              , identLetter = (alphaNum <|> char '_')
              , opStart = oneOf ":=<>"
              , opLetter = oneOf ":=<>"
              , reservedOpNames = ["=", "==", ":=", "<", ">"]
              , reservedNames = ["if", "then", "else", "end",
                                 "while", "do"]
              }

TokenParser{ parens = m_parens
           , identifier = m_identifier
           , integer = m_integer
           , float = m_float
           , reservedOp = m_reservedOp
           , reserved = m_reserved
           , semiSep1 = m_semiSep1
           , commaSep1 = m_commaSep1
           , commaSep = m_commaSep
           , comma = m_comma
           , semi = m_semi
           , whiteSpace = m_whiteSpace } = makeTokenParser def

exprParser :: Parser Expression
exprParser = buildExpressionParser table term <?> "expression"
table = [
  [Prefix (m_reservedOp "-" >> return (UnaExp UnaMinus))]
  , [Infix (m_reservedOp "+" >> return (BinExp Plus)) AssocLeft
    , Infix (m_reservedOp "-" >> return (BinExp Minus)) AssocLeft]
  , [Infix (m_reservedOp "==" >> return (BinExp EqOp)) AssocLeft]
  , [Infix (m_reservedOp "<" >> return (BinExp MLT)) AssocLeft
     , Infix (m_reservedOp ">" >> return (BinExp MGT)) AssocLeft]

  ]


posArgParser = do e <- exprParser 
                  return (PosArg e)

keyArgParser = do n <- m_identifier
                  m_reservedOp "="
                  v <- exprParser
                  return (KeyArg n v)

argParser = try (m_commaSep1 keyArgParser)
            <|> (do p <- (m_commaSep1 posArgParser)
                    v <- option [] (do m_semi
                                       k <- m_commaSep1 keyArgParser
                                       return k)
                    return (p ++ v))

-- term is the non-table part of the expression parser.
term = m_parens exprParser
       <|> try (do n <- m_identifier
                   args <- m_parens (option [] argParser)
                   return (ProcCallExp n args))
       <|> do n <- m_identifier
              return (VarExp n)
       <|> try (do f <- m_float
                   return (FloatExp f))
       <|> do i <- m_integer
              return (IntExp i)
       <|> do (char '"')
              s <- many (noneOf "\"")
              (char '"')
              return (StringExp s)


assignP = 
  do n <- m_identifier
     m_reservedOp ":="
     e <- exprParser
     return (Fix $ AssignmentStm n e)

ifP =
  do m_reserved "if"
     b <- exprParser
     m_reserved "then"
     p <- stmtParser
     m_reserved "else"
     q <- stmtParser
     m_reserved "end"
     return (Fix $ IfStm b p q)

whileP =
  do m_reserved "do"
     p <- stmtParser
     m_reserved "while"
     b <- exprParser
     return (Fix $ DoStm b p)

stmtParser :: Parser Statement
stmtParser = do ss <- m_semiSep1 stmt1 -- hard
                return (compoundStms ss)

stmt1 = ifP
        <|> whileP
        <|> try assignP
        <|> fmap (Fix . ExpressionStm) exprParser


mainParser :: Parser Statement
mainParser = m_whiteSpace >> stmtParser <* eof

parseString :: String -> IO ()
parseString inp =
  case parse mainParser "" inp of
    Left err -> print err
    ; Right ans -> print ans

parseFile fn =
  do t <- readFile fn
     return (parse mainParser "" t)
       
