module TreeGraph (
  treeToGraph, findPartner, findPartner'
  , findParent, findChildren, findSiblings
  , nthChild
  , findRoots
  , findNodes
  , orderedSibs
  , insertLeaf
  , deleteLeaf
  , moveTree
  , findMax
  , postOrderG
  , breadthFirstG
  , pathFromRoot
  ) where

import Debug.Trace -- because I'm stuck!
import qualified Data.Foldable as F
import Control.Monad
import Control.Exception
import Data.List as L
import qualified Data.Set as S
import Tree
import TupleGraph

-- Return list of (parent, child) pairs
treeToGraph :: Tree a -> [(a, a)]
treeToGraph (Node p children) =
  ([(p, parent c) | c <- children]
   ++ (concat $ map treeToGraph children))

findRoots :: Eq a => [(a, a)] -> [a]
findRoots g =
  let parents = nub $ map fst g
      children = nub $ map snd g
  in parents \\ children

findNodes :: Eq a => [(a, a)] -> [a]
findNodes g =
  let parents = map fst g
      children = map snd g
  in nub $ parents ++ children
  

pathFromRoot g e =
  let
    roots = findRoots g
    growUp l@(x:xs) = case x `elem` roots of
      True -> x:xs
      False ->
        let Just p = findParent x g
        in growUp (p:l)
  in
   growUp [e]
    
findPartner x matches = lookup x matches

-- findPartner' is 'partner1' in EderWiggs.
findPartner' y matches = fmap fst $ find (\t -> snd t==y) matches

findParent x g =
  case preds g x of
       [p] -> Just p
       _ -> Nothing

findChildren x g = succs g x

nthChild z n g =
  (findChildren z g) !! n

-- w00t! look at me, threading those monads!
findSiblings x g =
  do y <- findParent x g
     return $ findChildren y g

findMax g =
  let (max1, max2) =
        foldr1 (\(a,b) (m1, m2) -> (max a m1, max b m2)) g
  in max max1 max2

orderedSibs x g ordered =
  (reverse . F.concat . liftM (filter (\s -> s `S.member` ordered))) $ findSiblings x g

isLeaf :: (Eq a) => [(a,a)] -> a -> Bool
isLeaf g e =
  (e `notElem` (map fst g)) && (e `elem` (map snd g))

deleteChild g x = filter (\(p,c) -> c /= x) g

deleteLeaf :: (Show a, Eq a) => [(a,a)] -> a -> [(a,a)]
deleteLeaf g x =
  trace ("deleteLeaf: " ++  show x ++ "\n" ++ show g) assert (isLeaf g x) (deleteChild g x)

-- Add x as a new child of y, in the k-th position
insertAt :: (Eq a, Show a) => [(a,a)] -> a -> a -> Integer -> [(a,a)]
insertAt g x y k =
  trace (show g) $ 
  let
    insertAux ((p,c):gs) i -- precondition: k > 0; at least k-1 children of g at start.
      | p /= y               = trace (show i ++ " " ++ show p) $ (p,c):(insertAux gs i)
      | (p==y) && (i < k-1)  = (p,c):(insertAux gs (i+1))
      | (p==y) && (i == k-1) = (p,c):(p,x):gs
  in
   case k of
     0 -> (y,x):g
     _ -> insertAux g 0

-- Note - in the case that nodes are numbers (which is the only case I actually use)
-- the *caller* is still responsible for supplying the number.
insertLeaf g x y k = insertAt g x y k

-- move node x to the k-th child of y;
-- the whole subtree of x (if any) moves too.
moveTree :: (Eq a, Show a) => [(a,a)] -> a -> a -> Integer -> [(a,a)]
moveTree g x y k =
  let 
      g' = deleteChild g x
  in
   insertAt g' x y k

preOrderG g =
  let
    parents = nub $ map fst g
    children = nub $ map snd g
    [root] = findRoots g
  in
   preOrderAux root g
  where
    preOrderAux :: (Eq a) => a -> [(a, a)] -> [a]
    preOrderAux p g =
      let 
        children = findChildren p g 
        preC =  concat $ map (\c -> preOrderAux c g) children
      in
       p : preC

-- postOrderG is not unlike a top-sort...
postOrderG :: (Eq a) => [(a,a)] -> [a]
postOrderG g = reverse $ preOrderG g

breadthFirstG g =
  let
    [r] = findRoots g
  in
  breadthFirstAux [r] []
  where
    breadthFirstAux (n:ns) res =
      let
        children = (findChildren n g)
      in breadthFirstAux (ns ++ children) (n:res)
    breadthFirstAux [] res =
      reverse res
