module WriteGraph (
  makeEngineGraph
                  ) where

import Data.List as DL
import Control.Monad -- one thing for sure, there will be monads.
import Expression
import MyGraph (getNodes)
import FlowGraph
import Alpern -- labelArguments, Label, Node

type ArgName = String
type ArgVal = Expression

data EELabel = EEFuncL String [(ArgName, ArgVal)]
             | EEVarL String
             | EEExpL Expression
             deriving Show

type EENode = (Integer, Maybe String, EELabel)
type EEEdge = (String, Integer, Integer)
         
makeEngineGraph fn cfg = 
  let
    -- we only handle basic items:
    ns = DL.concat $ DL.map (extractItems . snd) $ getNodes cfg
    (i, vges, vgns) = DL.foldl (doNode fn) (0, [], []) ns
  in 
   (vges, vgns)

doNode fn (i, edges, nodes) n =
  let 
    definee = extractItemDefVar n :: Maybe String
    exp = extractItemExpression n :: Expression --  RHS of def or phi or bar expression
    (j, edges', nodes') = doExpression fn (i, edges, nodes) exp definee
  in
   (j, edges', nodes')

isMagicArgument function (argument, _) =
  argument == "vis" -- placeholder



doExpression :: String -> (Integer, [EEEdge], [EENode]) -> Expression -> Maybe String ->
                (Integer, [EEEdge], [EENode])
doExpression fn (i, edges, nodes) e definee =
  let j0 = i+1
      definee' = liftM ((fn++":")++) definee
  in case e of
    (ProcCallExp n args) ->
      let lArgs = labelArguments args
          (magic, notmagic) = partition (isMagicArgument n) lArgs
          nodes0 = (i, definee', EEFuncL n notmagic):nodes
          accn = DL.foldr (\(l, VarExp v) (j', edges', nodes') ->
                            let v' = (fn++":")++v
                                nodes'' = (j', Nothing, EEVarL v'):nodes'
                                edges'' = (l, i, j'):edges'
                            in (j'+1, edges'', nodes''))
                 (j0, edges, nodes0) magic
      in accn
    _ ->
      let n = EEExpL e
      in (j0, edges, (i, definee', n):nodes)
    
