import System.Exit
import Control.Monad
import Data.List as DL
import Data.Maybe
import Data.Either
import Data.Map as DM
import Data.Tuple
import qualified Data.Set as DS
import System.Environment
import ASTPlus
import Tree
import StmNode
import MyGraph
import TreeGraph
import TupleGraph
import FlowGraph
import MyGraph (getInt)
import ToyParser
import EncodedTree
import Domination
import SSA
import Alpern
import FromCache
import WriteGraph
import DMSUtil
import Debug.Trace -- because I'm stuck

main = do
  fns <- getArgs
  (js, fns, vgesList, vgns) <-
        foldM (\(is0, fns0, vges0, vgns0) fn -> do
                  (j, vges', vgns') <- filenameToValueGraph (head is0) fn
                  return (j:is0, fn:fns0, vges':vges0, vgns0++vgns'))
        ([0], [], [], []) fns
  let nodeRanges = reverse (zip fns $ DL.map swap $ pairwise js)
  let vges = foldl1 mergePLs vgesList
  putStrLn $ "All nodes\n" ++ showNewlineList vgns
  putStrLn $ "All edges\n" ++ showNewlineList vges
  let invfs = DL.map (\(edgeName, edges) -> (edgeName, DL.map swap edges)) vges
    -- The initial set of partitions is defined by those with the same
    -- "function".  But we have to interpret "function" in a broad
      -- sense: any "function label" (including eg Ints) are eligible.
  let fgroups = groupByFunctions vgns
  putStrLn $ "function groups:\n" ++ (showNewlineList fgroups)
  let (dupes, sings) = DL.partition ((>1) . length) $ hopcroftPartition fgroups invfs
  let singnodes = lookupNodeNums vgns sings
  putStrLn $ "Duplicated node numbers:" ++ show dupes
  let nodes = lookupNodeNums vgns dupes
  let varDupes = (DL.filter (isVarFromFundef . head) nodes)
  let fileNodes = matchFiles nodeRanges varDupes
  let lastFn = last fns
  let subs = makeSubstitutions fileNodes lastFn
  putStrLn $ "Ranges:\n" ++ showNewlineList nodeRanges
  putStrLn $ "Unduplicated nodes:\n" ++ showNewlineList singnodes
  putStrLn $ "Duplicated nodes (dupe nodes):\n" ++ showNewlineList nodes
  putStrLn $ "Duplicate variables:\n" ++ showNewlineList varDupes
  putStrLn $ "Filenames and vars:\n" ++ showNewlineList fileNodes
  putStrLn $ "Substitions:\n" ++ showNewlineList subs
  cfgAgain <- filenameToCFG 0 lastFn
  putStrLn $ "CFGAgain: " ++ show cfgAgain
  let cfg3 = substituteVars cfgAgain subs
  putStrLn $ "CFG3 " ++ show cfg3
  let (_, ves, vns) = makeValueGraph cfg3 0
  putStrLn $ "Value graph: " ++ show vns ++ "\nedges:\n" ++ show ves
  let (ges, gns) = makeEngineGraph lastFn cfg3
  putStrLn $ "Engine graph edges: " ++ show ges
  putStrLn $ "Engine graph nodes: " ++ showNewlineList gns
  


filenameToCFG i0 fn = do
  cfg0 <- filenameToCFG0 i0 fn
  cfg1 <- cfgToSSA cfg0
  return cfg1

filenameToCFG0 i0 fn = do
  parseResult <- parseFile fn
  when (isLeft parseResult)
    (do let (Left errr) = parseResult
        print errr
        exitWith $ ExitFailure 1)
  let (Right protoAST) = parseResult
  let ast = enumerateTreeV $ promoteTree protoAST ()
      nodeMap = extractNodeMap ast
      intTree = stripNodes $ treeifyAST ast
  putStrLn $ "first tree\n" ++ (show ast)
  putStrLn $ "nodeMap\n" ++ (show nodeMap)
  putStrLn $ "intTree\n" ++ (show intTree)
  -- treeToGraph (<- TreeGraph) makes a graph-style representation of the tree.
  let g1 = treeToGraph intTree
  -- EncodedTree is defined in FluriEdit, but it really needn't be.
      encT1 = EncodedTree{graph=g1, mapping=nodeMap, maxInt=TreeGraph.findMax g1}
  putStrLn $ "Graph1\n" ++ (show g1)
  putStrLn $ "nodeMap\n" ++ (show nodeMap)
  -- fromTreeGraph makes CFGs; it lives in FlowGraph module.
  let (cfg1, inOutMap, gSynMap) = fromTreeGraph g1 nodeMap
      edges1 = edges cfg1
      nodes1 = nodes cfg1
  putStrLn $ "cfg1:\n" ++ (show cfg1)
  putStrLn $ "nodes1:\n" ++ (show nodes1)
  putStrLn $ "edges1:\n" ++ (show edges1)
  putStrLn $ "inOutMap:\n" ++ (show inOutMap)
  putStrLn $ "gSynMap:\n" ++ (show gSynMap)
  let cfg2 = mergeEdges cfg1
  return cfg2

cfgToSSA cfg2 = do
  let edges2 = edges cfg2
      nodes2 = nodes cfg2
  -- putStrLn $ "nodes2:\n" ++ (showNewlineList nodes2)
  -- putStrLn $ "edges2: " ++ (show edges2)
  let vars = DL.map (\node@(i, n) -> (node, extractDefVar n)) nodes2
  putStrLn $ "Defined variables: " ++ (show vars)
  let allVars = (nub . DL.concat) $ DL.map snd vars
  putStrLn $ "All vars" ++ (show allVars)
  let varMapNode = DL.map (\v -> getA v vars) allVars
  putStrLn $ "varMapNode: " ++ (show varMapNode)
  let varMap = DL.map (\(v, l) -> (v, DL.map fst l)) varMapNode
  putStrLn $ "roots: " ++ (show $ roots edges2)
  let doms = case roots edges2 of
        [] -> DM.empty
        [root] -> DM.map DS.fromList $ DM.fromList $ invertLMap $ DM.toList $
                  DM.map DS.toList $ iterateDomination cfg2
  let idomt = DL.reverse $ DL.map swap $ domTree cfg2
  putStrLn $ "doms:\n" ++ show doms
  putStrLn $ "idomt:\n" ++ showNewlineList idomt
  let domF = domFront idomt edges2 doms :: Map Integer (DS.Set Integer)
  putStrLn $ "domF:\n" ++ (show $ DM.filter (not . DS.null) domF)
  let phis = insertPhis allVars varMap domF :: [(String, (DS.Set Integer))]
  putStrLn $ "phis:\n" ++ (show phis)
  let phis' = DL.map (\(n, s) -> (n, DS.toList s)) phis
  putStrLn $ "phis':\n" ++ (show phis')
  let phisNodes = DL.map (\(v, l) -> (v, DL.map (\e -> (e, lookupNode cfg2 e)) l)) phis'
  putStrLn $ "phisNodes:\n" ++ (show phisNodes)
  let cfg3 = DL.foldl (\cfgLocal (v,l) -> insertPhiNodes v l cfgLocal) cfg2 phis'
  putStrLn $ "cfg3:\n" ++ (show cfg3)
  let g4 = renameVariables allVars cfg3 idomt
  putStrLn $ "g4:\n" ++ (show g4)
  return g4

filenameToValueGraph i0 fn = do
  cfg2 <- filenameToCFG0 i0 fn
  putStrLn $ "cfg2:\n" ++ (show cfg2)
  g4 <- cfgToSSA cfg2
  let (last, vges, vgns) = makeValueGraph g4 i0
  putStrLn $ "value graph nodes:\n" ++ (showNewlineList vgns)
  putStrLn $ "value graph edges:\n" ++ (showNewlineList vges)
  putStrLn $ "Last node number:\n" ++ (show last)
  return (last, vges, vgns)

-- the [[a]] variable below is sets of equivalent variables.
-- this is "lookupNodeNumbers in the list of nodes and flesh them out into nodes.
lookupNodeNums :: Eq a => [(a, t, t1)] -> [[a]] -> [[(a, t, t1)]]
lookupNodeNums vgns nodeNums =
  DL.map (\l ->
           DL.map (\i -> lookupValGraphNode i vgns)
           l) nodeNums

lookupValGraphNode i nodes =
  fromJust $ find (\n@(j, _, _) -> i==j) nodes

getA v nodeVars =
  let occurences = catMaybes $
                   DL.map (\(i, vars) ->
                            case v `DL.elem` vars of
                              True -> Just i
                              _ -> Nothing) nodeVars
  in (v, occurences)

