module Tree (
  ExpF(..)
  ) where

{-# LANGUAGE DeriveFunctor, DeriveTraversable #-}

import Data.Fix
import Data.Foldable
import Data.Traversable
import Data.Monoid
import Control.Applicative

data ExpF a x = Lit a | Add x x | Sub x x | Mult x x
              deriving (Eq, Ord, Show)

instance Functor (ExpF a) where
  fmap _ (Lit a) = Lit a
  fmap f (Add x y) = Add (f x) (f y)
  fmap f (Sub x y) = Sub (f x) (f y)
  fmap f (Mult x y) = Mult (f x) (f y)

type Exp a = Fix (ExpF a)

fmapE :: (a->b) -> Exp a -> Exp b
fmapE f (Fix (Lit a)) = Fix $ Lit (f a)
fmapE f (Fix (Add l r)) = Fix $ Add (fmapE f l) (fmapE f r)
fmapE f (Fix (Sub l r)) = Fix $ Sub (fmapE f l) (fmapE f r)
fmapE f (Fix (Mult l r)) = Fix $ Mult (fmapE f l) (fmapE f r)

newtype Exp2 a = Exp2 {getExp :: Exp a} deriving (Show)

instance Functor Exp2 where
  fmap f (Exp2 e) = Exp2 $ fmapE f e

foldMapE f (Fix (Lit a)) = (f a)
foldMapE f (Fix (Add l r)) = foldMapE f l `mappend` foldMapE f r
foldMapE f (Fix (Sub l r)) = foldMapE f l `mappend` foldMapE f r
foldMapE f (Fix (Mult l r)) = foldMapE f l `mappend` foldMapE f r

instance Foldable Exp2 where
  foldMap f (Exp2 e) = foldMapE f e

traverseE f (Fix (Lit a))    = Fix <$> (Lit <$> (f a))
traverseE f (Fix (Add l r))  = Fix <$> (Add <$> traverseE f l <*> traverseE f r)
traverseE f (Fix (Sub l r))  = Fix <$> (Sub <$> traverseE f l <*> traverseE f r)
traverseE f (Fix (Mult l r)) = Fix <$> (Mult <$> traverseE f l <*> traverseE f r)

instance Traversable Exp2 where
  traverse f (Exp2 e) = Exp2 <$> traverseE f e


{-

let anExp = Fix (Add (Fix (Lit 3)) (Fix (Lit 4))) :: Exp Integer

getExp $ fmap (+1) $ Exp2 anExp
=> (Add (Lit 4) (Lit 5))

-- from Foldable:
toList $ Exp2 anExp
=> [3,4]

-}
