module Tree (
  TreeF(..),
  Tree(..),
  Tree2(..),
  ) where

{-# LANGUAGE DeriveFunctor, DeriveTraversable #-}

import Data.Fix
import Data.Foldable
import Data.Traversable
import Data.Monoid
import Control.Applicative



data TreeF a x = Empty | Leaf a | Node x a x
               deriving (Eq, Show, Functor, Foldable, Traversable)

left (Node l _ _) = l
val (Node _ a _) = a
right (Node _ _ r) = r

type Tree a = Fix (TreeF a)

fmapT f (Fix Empty) = Fix Empty
fmapT f (Fix (Leaf a)) = Fix $ Leaf (f a)
fmapT f (Fix (Node l a r)) = Fix $ Node (fmapT f l) (f a) (fmapT f r)

newtype Tree2 a = Tree2 {getTree :: Tree a} deriving (Show)

instance Functor Tree2 where
  fmap f (Tree2 t) = Tree2 $ fmapT f t

foldMapT f (Fix Empty) = mempty
foldMapT f (Fix (Leaf a)) = f a
foldMapT f (Fix (Node l k r)) = foldMapT f l `mappend` f k `mappend` foldMapT f r

instance Foldable Tree2 where
  foldMap f (Tree2 t) = foldMapT f t

traverseT f (Fix Empty) = pure (Fix Empty)
traverseT f (Fix (Leaf a)) = Fix <$> Leaf <$> (f a)
traverseT f (Fix (Node l k r)) = Fix <$> (Node <$> traverseT f l <*> f k <*> traverseT f r)

instance Traversable Tree2 where
  traverse f (Tree2 t) = Tree2 <$> traverseT f t

{-



-}
