module SwiftParser (
  parseString
  , parseFile
                   ) where

import Text.ParserCombinators.Parsec
import qualified Text.ParserCombinators.Parsec.Token as PT
import Debug.Trace -- because I'm stuck!

{-
Real World Haskell became annoying before it became useful, so I'm going to try plagiarising 
<http://www.haskell.org/haskellwiki/Parsing_expressions_and_statements> instead.
-}

import Control.Applicative((<*))
-- try can be Text.ParserCombinators.Parsec.try or Text.Parsec.try;
-- we use the former.
import Text.Parsec hiding (try)
import Text.Parsec.String
import Text.Parsec.Expr
import Text.Parsec.Token
import Text.Parsec.Language

data Program = Program [NSDec] [Import] [Stm]
             deriving (Show, Eq, Ord)

-- prefix URI? 
data NSDec = NSDec (Maybe String) String
           deriving (Show, Eq, Ord)

data Dimension = Dimension deriving (Show, Eq, Ord)
data Formal = Formal Type String [Dimension] (Maybe Expression)
            deriving (Show, Eq, Ord)
data Import = Import String deriving (Show, Eq, Ord)

data Stm = TypeDecStm String TypeDecSpec -- typeDecl
         | ProcDecStm [Formal] String [Formal] ProcBody -- procedureDecl
         | VarDecStm Bool Type [VarDecPart]
         | AppProcDecStm [Formal] String [Formal] [AppProfile] String [AppArg] -- appProcedureDecl 
         | IfStm Expression [Stm] [Stm]
         | ForEachStm String (Maybe String) Expression [Stm]
         | IterateStm String [Stm] Expression
         | SwitchStm Expression [CaseGroup]
         | AssignStm String Expression
         | AppendStm String Expression
         | ProcedureInvocationStm Expression
         | ProcedureCallStatAssignManyReturns [String] Expression -- can only be a procedure call, but that's not a 1st class type
         deriving (Show, Eq, Ord)

data TypeDecSpec = Opaque
                 | StructSpec [[StructField]]
                 | ATypeSpec Type
                 deriving (Show, Eq, Ord)

data AppProfile = AppProfile Expression Expression
                deriving (Show, Eq, Ord)



data StructField = StructField Type String [Dimension] deriving (Show, Eq, Ord)
data Case = CaseExp Expression | Default deriving (Show, Eq, Ord)
data CaseGroup = CaseGroup Case [Stm] deriving (Show, Eq, Ord)

data Expression = BinExp BinOp Expression Expression
                | UnaExp UnaOp Expression
                | IntExp Integer
                | FloatExp Double
                | StringExp String
                | BoolExp Bool
                | VarExp String
                | FunCallExp FunctionCall
                | ProcCallExp String [ParamV]
                | ArrayInitializerExp ArrayInitializer
                deriving (Show, Eq, Ord)
                    
data UnaOp = UnaMinus | UnaPlus
           deriving (Show, Eq, Ord)

data BinOp = MLT | MLE | MGE | MGT | Plus | Minus | Times | IDiv | FDiv | Mod
           | Or | And | EqOp | NotEq
           deriving (Show, Eq, Ord)

data ParamV = Param (Maybe String) Expression
             deriving (Show, Eq, Ord)


data Type = Type String [Maybe String]
          deriving (Show, Eq, Ord)

data Suffix = Suffix (Maybe Type)
              deriving (Show, Eq, Ord)

data ReturnParam = ReturnParamType Type String
                 | ReturnParamExp String (Maybe Expression)
                 deriving (Show, Eq, Ord)

-- Global flag, Type, Variables of that type

data VarDecPart = VarDecPart String [Maybe Type] (Maybe Expression)
                  deriving (Show, Eq, Ord)

type Identifier = String

data MapDec = MapDecStr String
              | MapDecList Identifier [MapParam]
              deriving (Show, Eq, Ord)

data MapParam = MapParam String Expression deriving (Show, Eq, Ord)
                

data ProcBody = AppSpecBody AppSpec | StatementBody [Stm]
              deriving (Show, Eq, Ord)
data AppSpec = AppSpec String [AppArg]
             deriving (Show, Eq, Ord)
data AppArg = AppExp Expression | AppStdio StdioArg
            deriving (Show, Eq, Ord)
data StdioArg = StdioArg Stdio Expression
              deriving (Show, Eq, Ord)
data Stdio = Stdin | Stdout | Stderr
           deriving (Show, Eq, Ord)

data ArrayInitializer = ArrayInitializer Expression Expression (Maybe Expression)
                      deriving (Show, Eq, Ord)

data FunctionCall = FunctionCallSensible String [Expression]
                  | FunctionCallFilename String 
                  deriving (Show, Eq, Ord)


{--

--}


def = emptyDef{commentStart = "/*"
              , commentEnd = "*/"
              , identStart = letter
              , identLetter = (alphaNum <|> char '_')
              -- , stringStart = "\""
              -- , stringEnd = "\""
              , opStart = oneOf "=<>@+-/%&!|:"
              , opLetter = oneOf "=<>@+-/%&!|:"
              , reservedOpNames = ["@", "+", "-", "/", "%/", "%%", "==", "!=",
                                   "<", "<=", ">", ">=", "<<", "=", ":=", "&&", "||", "!"] 
              , reservedNames = ["namespace", "import", "type", "app", "global",
                                 "if", "else", "foreach", "iterate", "switch",
                                 "case", "default", "stdin", "stdout", "stderr", "true", "false",
                                 "[", "]", "<", ">"]
              }

TokenParser{ parens = m_parens
           , braces = m_braces
           , brackets = m_brackets
           , identifier = m_identifier
           , integer = m_integer
           , float = m_float
           , reservedOp = m_reservedOp
           , reserved = m_reserved
           , semiSep1 = m_semiSep1
           , commaSep1 = m_commaSep1
           , commaSep = m_commaSep
           , comma = m_comma
           , colon = m_colon
           , semi = m_semi
           , stringLiteral = m_string
           , whiteSpace = m_whiteSpace } = makeTokenParser def

expression :: Parser Expression
expression = buildExpressionParser table term <?> "expression"
table = [
  [Prefix (m_reservedOp "-" >> return (UnaExp UnaMinus)),
   Prefix (m_reservedOp "+" >> return (UnaExp UnaPlus))],
  [Infix (m_reservedOp "*" >> return (BinExp Times)) AssocLeft,
   Infix (m_reservedOp "/" >> return (BinExp FDiv)) AssocLeft,
   Infix (m_reservedOp "%/" >> return (BinExp IDiv)) AssocLeft,
   Infix (m_reservedOp "%" >> return (BinExp Mod)) AssocLeft],
  [Infix (m_reservedOp "+" >> return (BinExp Plus)) AssocLeft,
   Infix (m_reservedOp "-" >> return (BinExp Minus)) AssocLeft],
  [Infix (m_reservedOp "<" >> return (BinExp MLT)) AssocLeft,
   Infix (m_reservedOp ">" >> return (BinExp MGT)) AssocLeft,
   Infix (m_reservedOp "<=" >> return (BinExp MLE)) AssocLeft,
   Infix (m_reservedOp ">" >> return (BinExp MGE)) AssocLeft],
  [Infix (m_reservedOp "!=" >> return (BinExp NotEq)) AssocLeft,
   Infix (m_reservedOp "==" >> return (BinExp EqOp)) AssocLeft],
  [Infix (m_reservedOp "||" >> return (BinExp Or)) AssocLeft],
  [Infix (m_reservedOp "&&" >> return (BinExp And)) AssocLeft]
  ]

-- term is the non-table part of the expression parser.
-- this corresponds to primExpr in swiftscript.g
term = m_parens expression
--       <|> try (do p <- procedureInvocationExpr
--                   return p)
       <|> do f <- functionInvocation
              return (FunCallExp f)
       <|> do n <- m_identifier
              return (VarExp n)
       <|> constant
       

constant = try (do f <- m_float
                   return (FloatExp f))
           <|> do i <- m_integer
                  return (IntExp i)
           <|> (m_reserved "true" >> return (BoolExp True))
           <|> (m_reserved "false" >> return (BoolExp False))
           <|> do s <- m_string
                  return (StringExp s)
           <|> do ai <- arrayInitializer
                  return $ ArrayInitializerExp ai

mainParser = m_whiteSpace >> programParser <* eof


-- note: "many" means zero or more instances.
-- (There's also many1 for one or more.)
programParser = do ns <- many nsdecl
                   imports <- many importStatement
                   topStms <- many topLevelStatement
                   return (Program ns imports topStms)


nsdecl = do (m_reserved "namespace")
            prefix <- option Nothing $ do i <- m_identifier
                                          return (Just i)
            uri <- m_string
            m_semi
            return (NSDec prefix uri)

importStatement =  do (m_reserved "import")
                      importee <- m_string
                      return $ Import importee

topLevelStatement = typeDecl
                    <|> try procedureDecl
                    <|> try appProcedureDecl
                    <|> innerStatement


typeDecl =  do (m_reserved "type")
               t <- m_identifier
               typeSpec <- do m_semi
                              return Opaque
                           <|> do t <- aType
                                  m_semi
                                  return $ ATypeSpec t
                           <|> do s <- structDecl
                                  return $ StructSpec s
               return $ TypeDecStm t typeSpec

structDecl = 
  do fields <- m_braces $ many $ do sf <- m_commaSep (do t <- aType
                                                         n <- m_identifier
                                                         dims <- many (do m_reserved "["; m_reserved "]"; return Dimension)
                                                         return $ StructField t n dims)
                                    m_semi
                                    return sf
     optional (m_semi)
     return fields
                         

innerStatement = try (varDeclaration)
                 <|> nonDecInnerStatement

nonDecInnerStatement = ll1statement
                       <|> try (do p <- procedureInvocationExpr
                                   m_semi
                                   return $ ProcedureInvocationStm p)
                       <|> try assignStat 
                       <|> try appendStat
                       <|> try (procedureCallStatAssignManyReturnParam)
                          
varDeclaration = do global <- option False (do m_reserved "global"
                                               return True)
                    t <- aType
                    parts <- m_commaSep1 varDeclPart
                    m_semi
                    return $ VarDecStm global t parts

varDeclPart = do name <- m_identifier
                 suffixes <- many $ m_brackets $ option Nothing $ do t <- aType
                                                                     return $ Just t
                 option Nothing $ do m_reserved "<"
                                     d <- mappingDecl
                                     m_reserved ">"
                                     return $ Just d
                 mval <- option Nothing $ do m_reservedOp ":="
                                             val <- expression
                                             return $ Just val
                 return $ VarDecPart name suffixes mval

arrayInitializer =
  m_brackets $ try (do from <- expression
                       m_colon
                       to <- expression
                       step <- option Nothing (do m_colon
                                                  e <- expression
                                                  return $ Just e)
                       return $ ArrayInitializer from to step)

mappingDecl =  (do s <- m_string
                   return $ MapDecStr s)
               <|>
               (do n <- m_identifier
                   m_semi
                   l <- option [] $ m_commaSep mapParam
                   return $ MapDecList n l)

mapParam = do name <- m_identifier
              m_reservedOp "="
              value <- expression
              return $ MapParam name value


procedureDecl =
  do formalsOut <- option [] $ m_parens $ m_commaSep1 formalParameter
     name <- m_identifier
     formalsIn <- m_parens $ m_commaSep formalParameter
     body <- m_braces procBody
     return $ ProcDecStm formalsOut name formalsIn body 



procBody = (do a <- appSpec
               return $ AppSpecBody a)
           <|>
           (do s <- many innerStatement
               return $ StatementBody s)

appProcedureDecl = do m_reserved "app"
                      paramsOut <- option [] $ m_parens $ m_commaSep1 formalParameter
                      name <- m_identifier
                      paramsIn <- m_parens $ m_commaSep formalParameter
                      (profiles', pname', args')  <- m_braces $ do profiles <- many appProfile
                                                                   pname <- m_identifier
                                                                   args <- many appArg
                                                                   m_semi
                                                                   return (profiles, pname, args)
                      return $ AppProcDecStm paramsOut name paramsIn profiles' pname' args'


appProfile = do m_reserved "profile"
                e1 <- expression
                m_reservedOp "="
                e2 <- expression
                m_semi
                return $ AppProfile e1 e2

appArg = (do e <- expression
             return $ AppExp e)
         <|>
         (do s <- stdioArg
             return $ AppStdio s)
         
-- I am not convinced this is right, but/and it is also deprecated

appSpec = do m_reserved "app"
             m_braces $ do name <- m_identifier
                           args <- many appArg
                           m_semi
                           return $ AppSpec name args

formalParameter = do t <- aType
                     name <- m_identifier
                     dims <- many $ do {m_reserved "["; m_reserved "]"; return Dimension}
                     val <- option Nothing $ do m_reservedOp "="
                                                c <- constant
                                                return $ Just c
                     return $ Formal t name dims val
                                         
aType = do typename <- m_identifier
           subs <- many typeSubscript
           return $ Type typename subs

typeSubscript = m_brackets $ option Nothing $ do id <- m_identifier
                                                 return $ Just id

compoundStat = m_brackets $ many innerStatement

ll1statement = ifStat <|> foreachStat <|> switchStat <|> iterateStat

  
ifStat = do m_reserved "if"
            e <- m_parens expression
            s <- compoundStat
            me <- option [] $ do m_reserved "else"
                                 e <- compoundStat
                                 return e
            return $ IfStm e s me

foreachStat = do m_reserved "foreach"
                 v <- m_identifier
                 ind <- option Nothing $ do m_comma
                                            index <- m_identifier
                                            return $ Just index
                 m_reserved "in"
                 e <- expression
                 s <- compoundStat
                 return $ ForEachStm v ind e s

iterateStat = do m_reserved "iterate"
                 i <- m_identifier
                 s <- compoundStat
                 m_reserved "until"
                 e <- m_parens expression
                 m_semi
                 return $ IterateStm i s e

switchStat = do m_reserved "switch"
                e <- m_parens expression
                s <- m_braces $ many casesGroup
                return $ SwitchStm e s

casesGroup = do c <- (do m_reserved "case"
                         e <- expression
                         return (CaseExp e)
                      <|>
                      do m_reserved "default"
                         return Default)
                m_colon
                s <- many nonDecInnerStatement
                return $ CaseGroup c s


-- The fullExpression type used here is unusual in that these are the
-- only two places where procedure calls are allowed.  You're NOT
-- allowed "f(g(x))" - the result of every procedure call must be
-- stored in a variable or array.

-- This is surely by design, although I haven't seen them say so
-- anywhere.

assignStat = do n <- m_identifier
                m_reservedOp "="
                v <- fullExpression 
                m_semi
                return $ AssignStm n v

appendStat = do n <- m_identifier
                m_reservedOp "<<"
                v <- fullExpression
                m_semi
                return $ AppendStm n v

fullExpression = procedureInvocationExpr
                 <|> expression

procedureInvocationExpr = do procname <- m_identifier
                             params <- m_parens $ m_commaSep actualParameter
                             return $ ProcCallExp procname params

actualParameter = do mn <- option Nothing (try (do name <- m_identifier
                                                   m_reservedOp "="
                                                   return $ Just name))
                     e <- expression
                     return $ Param mn e

-- this is a bit of fail-boating: multiple assignment needs its own
-- grammar rule because tuples are not otherwise part of the language.
procedureCallStatAssignManyReturnParam =
  do names <- m_parens $ m_commaSep1 $ m_identifier
     m_reservedOp "="
     expr <- procedureInvocationExpr
     m_semi
     return $ ProcedureCallStatAssignManyReturns names expr

-- returnParameter is the variable to which one of a procedure's
-- multiple return values will be assigned.  Note that this is quite
-- different from the single result case (which does not allow a type
-- or a default value).
returnParameter = (do atype <- aType
                      name <- m_identifier
                      return $ ReturnParamType atype name
                  )
                  <|>
                  (do name <- m_identifier
                      exp <- option Nothing $ do m_reservedOp "="
                                                 e <- expression
                                                 return $ Just e
                      return $ ReturnParamExp name exp)

stdioArg = do s <- ((m_reserved "stdin" >> (return Stdin))
                    <|> (m_reserved "stdout" >> (return Stdout))
                    <|> (m_reserved "stderr" >> (return Stderr)))
              m_reservedOp "="
              e <- expression
              return $ StdioArg s e

functionInvocation = do m_reservedOp "@"
                        (do n <- m_identifier
                            exps <- option [] $ m_parens $ m_commaSep expression
                            return $ FunctionCallSensible n exps)
                        -- the second case corresponds to "filename" somehow
                        <|> do n <- m_identifier
                               return $ FunctionCallFilename n
                        <|> do n <- m_parens m_identifier
                               return $ FunctionCallFilename n


parseString :: String -> IO ()
parseString inp =
  case parse mainParser "" inp of
    Left err -> print err
    ; Right ans -> print ans

parseFile fn =
  do t <- readFile fn
     return (parse mainParser "" t)
       
