-----------------------------------------------------------------------------
-- |
-- Copyright   : 2013 University of Cambridge
-- License     : BSD-3
--
-- Parser for Recipe, this version using parsec. The language concept
-- and parsing is designed to be similar to
-- /home/bnikolic/d/p/sources/exm-0.5.0/stc/src/exm/stc/ast/ExM.g
-----------------------------------------------------------------------------

module Language.Recipe.ParserParsec where

import Text.ParserCombinators.Parsec
import qualified Text.ParserCombinators.Parsec.Token as P
import Text.ParserCombinators.Parsec.Language (haskellDef, emptyDef)

import Language.Recipe.AST

recipeDef = emptyDef

lexer = P.makeTokenParser  recipeDef

-- See http://hackage.haskell.org/package/parsec-3.1.4/docs/Text-Parsec-Token.html#v:makeTokenParser
whiteSpace = P.whiteSpace lexer        
decimal    = P.decimal lexer
dot        = P.dot lexer
parens     = P.parens lexer
braces     = P.braces lexer
stringLiteral = P.stringLiteral lexer
integer = P.integer lexer
semi = P.semi lexer

runLex :: Show a => Parser a -> String -> IO ()
runLex p input
  = parseTest (do{ whiteSpace
           ; x <- p
           ; eof
           ; return x
           }) input



varName =   do {
  i <- (P.identifier lexer) <?> "Variable name";
  return (Ident i)}

typeName =  do {
  x <- (P.identifier lexer) ;
  return $ OpaqueType x;
  }

typePrefix = typeName

multiTypePrefix = typePrefix
             
argDecl   = do {
  t <- multiTypePrefix;
  i <- varName;
  return $ FormalParam t  i}

newTypeDef = do
  (P.symbol lexer "TYPE")
  i <- (P.identifier lexer);
  semi;
  return $ OpaqueTypeDef i;

formalParams = parens  $ do 
  x<- P.commaSep lexer argDecl <?> "Formal parameter list"
  return $ x ;

exprArgList = parens $ do
  x<- P.commaSep lexer baseExpr;
  return $ x

pyCmd = do
  P.symbol lexer "PYCMD";
  b <- stringLiteral;
  al <- exprArgList;
  semi;
  return $ PythonCall b al;

stmt =
  try pyCmd
  <|>
  try declAssign
  <|>
  try functionCall
  
functionDefinition = do
  (P.symbol lexer "APP")
  o <- formalParams
  n <- varName;
  p <- formalParams
  b <- (braces $ many stmt)
  return $ AppFunDefn o n p b

functionCall = do
  i <- (P.identifier lexer)
  al <- exprArgList
  return $ FunCall (Ident i) al

functionCallExp = do
  i <- (P.identifier lexer)
  al <- exprArgList <?> "Function call expression"
  return $ FunCallExp (Ident i) al  

baseExpr = do { l <- (try literal); return $ Lit l}  <|>
           do { i <- (try functionCallExp); return i} <|>
           do { i <- (try varName); return $  ExpName i} 


literal = (do {s <-stringLiteral;  return $ String s } ) <|> (do {i<- integer ; return $ Int i})

declAssign = do {
  t <- multiTypePrefix;
  n <- varName;
  P.lexeme lexer (char '=');
  v <- baseExpr ;
  semi;
  return $ DeclAssign t n v;
  }

tLevel= do { r <- (try stmt); return $ Stmt r;} <|>
        do { r <- (try newTypeDef) ; return $ TypeDef r;} <|>
        do {r <- functionDefinition; return $ FunDef r;}

program = many1 tLevel 
       
